@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 2016, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
@@ -7,7 +7,6 @@
 #include <stdarg.h>
 #include "nc4internal.h"
 #include "ncoffsets.h"
-#include "ezxml.h"
 
 /**
  * Build the netcdf-4 metadata from the NCD4node nodes.
@@ -64,14 +63,15 @@ NCD4_metabuild(NCD4meta* metadata, int ncid)
     metadata->root->meta.id = ncid;
 
     /* Fix up the atomic types */
-    for(i=0;i<nclistlength(metadata->allnodes);i++) {
-	NCD4node* n = (NCD4node*)nclistget(metadata->allnodes,i);
+    for(i=0;i<nclistlength(metadata->atomictypes);i++) {
+	NCD4node* n = (NCD4node*)nclistget(metadata->atomictypes,i);
 	if(n->sort != NCD4_TYPE) continue;
 	if(n->subsort > NC_MAX_ATOMIC_TYPE) continue;
 	n->meta.id = n->subsort;
         n->meta.isfixedsize = (n->subsort == NC_STRING ? 0 : 1);
 	if(n->subsort <= NC_STRING)
 	    n->meta.dapsize = NCD4_typesize(n->subsort);
+	n->container = metadata->root;
     }
 
     /* Topo sort the set of all nodes */
@@ -95,19 +95,27 @@ done:
 */
 
 NCD4meta*
-NCD4_newmeta(size_t rawsize, void* rawdata)
+NCD4_newmeta(NCD4INFO* info)
 {
     NCD4meta* meta = (NCD4meta*)calloc(1,sizeof(NCD4meta));
     if(meta == NULL) return NULL;
     meta->allnodes = nclistnew();
-    meta->serial.rawsize = rawsize;
-    meta->serial.rawdata = rawdata;
 #ifdef D4DEBUG
     meta->debuglevel = 1;
 #endif
+    meta->controller = info;
+    meta->ncid = info->substrate.nc4id; /* Transfer netcdf ncid */
     return meta;
 }
 
+/* Attach raw data to metadata */
+void
+NCD4_attachraw(NCD4meta* meta, size_t rawsize, void* rawdata)
+{
+    assert(meta != NULL);
+    NCD4_resetSerial(&meta->serial,rawsize,rawdata);
+}
+
 void
 NCD4_setdebuglevel(NCD4meta* meta, int debuglevel)
 {
@@ -119,16 +127,27 @@ NCD4_reclaimMeta(NCD4meta* dataset)
 {
     int i;
     if(dataset == NULL) return;
+    NCD4_resetMeta(dataset);
+
     for(i=0;i<nclistlength(dataset->allnodes);i++) {
 	NCD4node* node = (NCD4node*)nclistget(dataset->allnodes,i);
 	reclaimNode(node);
-    } 
-    nullfree(dataset->error.parseerror);
-    nullfree(dataset->error.message);
-    nullfree(dataset->error.context);
-    nullfree(dataset->error.otherinfo);
-    nullfree(dataset->serial.errdata);
+    }
+    nclistfree(dataset->allnodes);
     nclistfree(dataset->groupbyid);
+    nclistfree(dataset->atomictypes);
+    free(dataset);
+}
+
+void
+NCD4_resetMeta(NCD4meta* dataset)
+{
+    if(dataset == NULL) return;
+    nullfree(dataset->error.parseerror); dataset->error.parseerror = NULL;
+    nullfree(dataset->error.message); dataset->error.message = NULL;
+    nullfree(dataset->error.context); dataset->error.context = NULL;
+    nullfree(dataset->error.otherinfo); dataset->error.otherinfo = NULL;
+    NCD4_resetSerial(&dataset->serial,0,NULL);
 #if 0
     for(i=0;i<nclistlength(dataset->blobs);i++) {
 	void* p = nclistget(dataset->blobs,i);
@@ -136,31 +155,29 @@ NCD4_reclaimMeta(NCD4meta* dataset)
     }
     nclistfree(dataset->blobs);
 #endif
-    nclistfree(dataset->allnodes);
-    nullfree(dataset->serial.dmr);
-    free(dataset);
 }
 
 void
 reclaimNode(NCD4node* node)
 {
     if(node == NULL) return;
-    nullfree(node->name);
-    nclistfree(node->groups);
-    nclistfree(node->vars);
-    nclistfree(node->types);
-    nclistfree(node->dims);
-    nclistfree(node->attributes);
-    nclistfree(node->maps);
-    nclistfreeall(node->xmlattributes);
-    nclistfreeall(node->attr.values);
-    nclistfree(node->en.econsts);
-    nclistfree(node->group.elements);
-    nullfree(node->group.dapversion);
-    nullfree(node->group.dmrversion);
-    nullfree(node->group.datasetname);
-    nclistfree(node->group.varbyid);
-    nullfree(node->nc4.orig.name);
+    nullfree(node->name); node->name = NULL;
+    nclistfree(node->groups); node->groups = NULL;
+    nclistfree(node->vars); node->vars = NULL;
+    nclistfree(node->types); node->types = NULL;
+    nclistfree(node->dims); node->dims = NULL;
+    nclistfree(node->attributes); node->attributes = NULL;
+    nclistfreeall(node->mapnames); node->mapnames = NULL;
+    nclistfree(node->maps); node->maps = NULL;
+    nclistfreeall(node->xmlattributes); node->xmlattributes = NULL;
+    nclistfreeall(node->attr.values); node->attr.values = NULL;
+    nclistfree(node->en.econsts); node->en.econsts = NULL;
+    nclistfree(node->group.elements); node->group.elements = NULL;
+    nullfree(node->group.dapversion); node->group.dapversion = NULL;
+    nullfree(node->group.dmrversion); node->group.dmrversion = NULL;
+    nullfree(node->group.datasetname); node->group.datasetname = NULL;
+    nclistfree(node->group.varbyid); node->group.varbyid = NULL;
+    nullfree(node->nc4.orig.name); node->nc4.orig.name = NULL;
     nullfree(node);
 }
 
@@ -193,7 +210,7 @@ build(NCD4meta* builder, NCD4node* root)
 	case NC_STRUCT:
 	    /* We need to compute the field offsets in order to compute the struct size */
 	    computeOffsets(builder,x);
-	    break;		
+	    break;
         }
     }
 
@@ -279,7 +296,7 @@ buildGroups(NCD4meta* builder, NCD4node* parent)
     fprintf(stderr,"build group: %s\n",parent->name);
 #endif
     /* Define any group level attributes */
-    if((ret = buildAttributes(builder,parent))) goto done;    
+    if((ret = buildAttributes(builder,parent))) goto done;
 
     for(i=0;i<nclistlength(parent->groups);i++) {
 	NCD4node* g = (NCD4node*)nclistget(parent->groups,i);
@@ -302,7 +319,7 @@ buildDimension(NCD4meta* builder, NCD4node* dim)
     NCD4node* group = NCD4_groupFor(dim);
     if(dim->dim.isunlimited) {
 	NCCHECK((nc_def_dim(group->meta.id,dim->name,NC_UNLIMITED,&dim->meta.id)));
-    } else {    
+    } else {
 	NCCHECK((nc_def_dim(group->meta.id,dim->name,(size_t)dim->dim.size,&dim->meta.id)));
     }
 done:
@@ -315,7 +332,7 @@ buildEnumeration(NCD4meta* builder, NCD4node* en)
     int i,ret = NC_NOERR;
     NCD4node* group = NCD4_groupFor(en);
     NCCHECK((nc_def_enum(group->meta.id,en->basetype->meta.id,en->name,&en->meta.id)));
-    for(i=0;i<nclistlength(en->en.econsts);i++) {	
+    for(i=0;i<nclistlength(en->en.econsts);i++) {
 	NCD4node* ec = (NCD4node*)nclistget(en->en.econsts,i);
 	NCCHECK((nc_insert_enum(group->meta.id, en->meta.id, ec->name, ec->en.ecvalue.i8)));
     }
@@ -334,7 +351,7 @@ buildOpaque(NCD4meta* builder, NCD4node* op)
     /* Two cases, with and without UCARTAGORIGTYPE */
     if(op->nc4.orig.name != NULL) {
 	name = op->nc4.orig.name;
-	group = op->nc4.orig.group;		
+	group = op->nc4.orig.group;
     }
     NCCHECK((nc_def_opaque(group->meta.id,op->opaque.size,name,&op->meta.id)));
 done:
@@ -384,8 +401,8 @@ static int
 buildMetaData(NCD4meta* builder, NCD4node* var)
 {
     int ret = NC_NOERR;
-    if((ret = buildAttributes(builder,var))) goto done;    
-    if((ret = buildMaps(builder,var))) goto done;    
+    if((ret = buildAttributes(builder,var))) goto done;
+    if((ret = buildMaps(builder,var))) goto done;
 done:
     return THROW(ret);
 }
@@ -437,6 +454,10 @@ buildAttributes(NCD4meta* builder, NCD4node* varorgroup)
 	if(strncmp(attr->name,UCARTAG,strlen(UCARTAG)) == 0)
 	    continue;
 
+	/* Suppress all reserved attributes */
+	if(NCD4_lookupreserved(attr->name) != NULL)
+	    continue;
+
 	if(ISGROUP(varorgroup->sort))
 	    varid = NC_GLOBAL;
 	else
@@ -476,7 +497,7 @@ buildStructureType(NCD4meta* builder, NCD4node* structtype)
     /* Step 2: See if already defined */
     if(nc_inq_typeid(group->meta.id,name,&tid) == NC_NOERR) {/* Already exists */
 	FAIL(NC_ENAMEINUSE,"Inferred type name conflict",name);
-    }    
+    }
 
     /* Since netcdf does not support forward references,
        we presume all field types are defined */
@@ -509,7 +530,7 @@ buildVlenType(NCD4meta* builder, NCD4node* vlentype)
     /* See if already defined */
     if(nc_inq_typeid(group->meta.id,name,&tid) == NC_NOERR) {/* Already exists */
 	FAIL(NC_ENAMEINUSE,"Inferred type name conflict",name);
-    }    
+    }
 
     /* Get the baseline type */
     basetype = vlentype->basetype;
@@ -533,7 +554,7 @@ buildCompound(NCD4meta* builder, NCD4node* cmpdtype, NCD4node* group, char* name
     NCCHECK((nc_def_compound(group->meta.id,(size_t)cmpdtype->meta.memsize,name,&cmpdtype->meta.id)));
 
     /* Step 3: add the fields to type */
-    for(i=0;i<nclistlength(cmpdtype->vars);i++) {  
+    for(i=0;i<nclistlength(cmpdtype->vars);i++) {
 	int rank;
 	int dimsizes[NC_MAX_VAR_DIMS];
         NCD4node* field = (NCD4node*)nclistget(cmpdtype->vars,i);
@@ -579,7 +600,7 @@ buildAtomicVar(NCD4meta* builder, NCD4node* var)
     savevarbyid(group,var);
 
     /* Build attributes and map attributes */
-    if((ret = buildMetaData(builder,var))) goto done;    
+    if((ret = buildMetaData(builder,var))) goto done;
 done:
     return THROW(ret);
 }
@@ -601,7 +622,7 @@ buildStructure(NCD4meta* builder, NCD4node* structvar)
     savevarbyid(group,structvar);
 
     /* Build attributes and map attributes WRT the variable */
-    if((ret = buildMetaData(builder,structvar))) goto done;    
+    if((ret = buildMetaData(builder,structvar))) goto done;
 
 done:
     return THROW(ret);
@@ -623,7 +644,7 @@ buildSequence(NCD4meta* builder, NCD4node* seq)
     savevarbyid(group,seq);
 
     /* Build attributes and map attributes WRT the variable */
-    if((ret = buildMetaData(builder,seq))) goto done;    
+    if((ret = buildMetaData(builder,seq))) goto done;
 
 done:
     return THROW(ret);
@@ -652,7 +673,7 @@ savevarbyid(NCD4node* group, NCD4node* var)
     nclistinsert(group->group.varbyid,var->meta.id,var);
 }
 
-/* Collect FQN path from node upto (but not including)
+/* Collect FQN path from node up to (but not including)
    the first enclosing group and create an name from it
 */
 static char*
@@ -682,7 +703,7 @@ getFieldFQN(NCD4node* field, const char* tail)
         ncbytescat(fqn,tail);
     result = ncbytesextract(fqn);
     ncbytesfree(fqn);
-    return result;    
+    return result;
 }
 
 static size_t
@@ -755,7 +776,7 @@ compileAttrValues(NCD4meta* builder, NCD4node* attr, void** memoryp, NClist* blo
 		/* Force type match */
 		basetype = (attr->basetype = container->basetype);
 	    } else {/* Fail */
-	        FAIL(NC_EBADTYPE,"_FillValue/Variable type mismatch: %s:%s",container->name,attr->name);		
+	        FAIL(NC_EBADTYPE,"_FillValue/Variable type mismatch: %s:%s",container->name,attr->name);
 	    }
 	}
     }
@@ -830,7 +851,11 @@ convertString(union ATOMICS* converter, NCD4node* type, const char* s)
 	if(sscanf(s,"%lf",&converter->f64[0]) != 1) return THROW(NC_ERANGE);
 	break;
     case NC_CHAR:
+#ifdef WORDS_BIGENDIAN
+	converter->i8[7] = s[0];
+#else
 	converter->i8[0] = s[0];
+#endif
 	break;
     case NC_STRING:
 	converter->s[0]= strdup(s);
@@ -962,7 +987,7 @@ markfixedsize(NCD4meta* meta)
 	if(n->sort != NCD4_TYPE) continue;
 	switch (n->subsort) {
 	case NC_STRUCT:
-            for(j=0;j<nclistlength(n->vars);j++) {  
+            for(j=0;j<nclistlength(n->vars);j++) {
                 NCD4node* field = (NCD4node*)nclistget(n->vars,j);
 	        if(!field->basetype->meta.isfixedsize) {
 		    fixed = 0;
@@ -973,7 +998,7 @@ markfixedsize(NCD4meta* meta)
 	    break;
 	case NC_ENUM:
 	    n->meta.isfixedsize = 1;
-	    break;	
+	    break;
 	default: /* leave as is */
 	    break;
 	}
@@ -1009,36 +1034,38 @@ computeOffsets(NCD4meta* builder, NCD4node* cmpd)
 	} else if(ftype->subsort == NC_SEQ) { /* VLEN */
 	    alignment = nctypealignment(NC_VLEN);
 	    assert(ftype->meta.memsize > 0); size=ftype->meta.memsize;
-	    //size = NCD4_computeTypeSize(builder,ftype);
+	    /*size = NCD4_computeTypeSize(builder,ftype);*/
 	} else if(ftype->subsort == NC_OPAQUE) {
 	    /* Either fixed or a vlen */
 	    assert(ftype->meta.memsize > 0); size=ftype->meta.memsize;
 	    if(ftype->opaque.size == 0) {/* treat like vlen */
 	        alignment = nctypealignment(NC_VLEN);
-	        //size = NCD4_computeTypeSize(builder,ftype);
+	        /*size = NCD4_computeTypeSize(builder,ftype);*/
 	    } else { /* fixed size */
 	        alignment = nctypealignment(NC_OPAQUE);
-	        //size = NCD4_computeTypeSize(builder,ftype);
+	        /*size = NCD4_computeTypeSize(builder,ftype);*/
 	    }
 	} else if(ftype->subsort == NC_ENUM) {
 	    NCD4node* truetype = ftype->basetype;
 	    alignment = nctypealignment(truetype->meta.id);
 	    assert(ftype->meta.memsize > 0); size=ftype->meta.memsize;
-	    //size = NCD4_computeTypeSize(builder,truetype);
+	    /*size = NCD4_computeTypeSize(builder,truetype);*/
 	} else { /* Basically a primitive */
 	    alignment = nctypealignment(ftype->meta.id);
 	    assert(ftype->meta.memsize > 0); size=ftype->meta.memsize;
-	    //size = NCD4_computeTypeSize(builder,ftype);
+	    /*size = NCD4_computeTypeSize(builder,ftype);*/
 	}
 #endif
         if(alignment > largestalign)
 	    largestalign = alignment;
 	/* Add possible padding wrt to previous field */
-	offset += getpadding(offset,alignment);	
+	offset += getpadding(offset,alignment);
 	field->meta.offset = offset;
 	assert(ftype->meta.memsize > 0);
 	size = ftype->meta.memsize;
-	//field->meta.memsize = size;
+#if 0
+	field->meta.memsize = size;
+#endif
 	/* Now ultiply by the field dimproduct*/
 	if(nclistlength(field->dims) > 0) {
             d4size_t count = NCD4_dimproduct(field);
@@ -1086,7 +1113,7 @@ NCD4_computeTypeSize(NCD4meta* builder, NCD4node* type)
         }
         break;
     default: break; /* ignore */
-    }        
+    }
     type->meta.memsize = size;
     return size;
 }
@@ -1115,7 +1142,7 @@ markdapsize(NCD4meta* meta)
 	switch (type->subsort) {
 	case NC_STRUCT:
 	    totalsize = 0;
-            for(j=0;j<nclistlength(type->vars);j++) {  
+            for(j=0;j<nclistlength(type->vars);j++) {
                 NCD4node* field = (NCD4node*)nclistget(type->vars,j);
 		size_t size = field->basetype->meta.dapsize;
 	        if(size == 0) {
@@ -1128,17 +1155,17 @@ markdapsize(NCD4meta* meta)
 	    break;
 	case NC_SEQ:
 	    type->meta.dapsize = 0; /* has no fixed size */
-	    break;	
+	    break;
 	case NC_OPAQUE:
 	    type->meta.dapsize = type->opaque.size;
-	    break;	
+	    break;
 	case NC_ENUM:
 	    type->meta.dapsize = type->basetype->meta.dapsize;
-	    break;	
+	    break;
 	case NC_STRING:
 	    type->meta.dapsize = 0; /* has no fixed size */
-	    break;		
-	default: 
+	    break;
+	default:
 	    assert(type->subsort <= NC_UINT64);
 	    /* Already assigned */
 	    break;
@@ -1147,3 +1174,31 @@ markdapsize(NCD4meta* meta)
     return NC_NOERR;
 }
 
+int
+NCD4_findvar(NC* ncp, int ncid, int varid, NCD4node** varp, NCD4node** grpp)
+{
+    int ret = NC_NOERR;
+    NCD4INFO* info = NULL;
+    NCD4meta* meta = NULL;
+    NCD4node* var = NULL;
+    NCD4node* group = NULL;
+    int grp_id;
+
+    info = getdap(ncp);
+    if(info == NULL)
+	return THROW(NC_EBADID);
+    meta = info->substrate.metadata;
+    if(meta == NULL)
+	return THROW(NC_EBADID);
+    /* Locate var node via (grpid,varid) */
+    grp_id = GROUPIDPART(ncid);
+    group = nclistget(meta->groupbyid,grp_id);
+    if(group == NULL)
+	return THROW(NC_EBADID);
+    var = nclistget(group->group.varbyid,varid);
+    if(var == NULL)
+	return THROW(NC_EBADID);
+    if(varp) *varp = var;
+    if(grpp) *grpp = group;
+    return ret;
+}
