@@ -1,4 +1,4 @@
-/* Copyright 2003-2018, University Corporation for Atmospheric
+/* Copyright 2003-2022, University Corporation for Atmospheric
  * Research. See COPYRIGHT file for copying and redistribution
  * conditions. */
 /**
@@ -11,13 +11,21 @@
 
 #include "config.h"
 #include "hdf5internal.h"
+#include "hdf5err.h"
+#include "hdf5debug.h"
 #include "ncrc.h"
+#include "ncauth.h"
 #include "ncmodel.h"
+#include "ncpathmgr.h"
 
 #ifdef ENABLE_BYTERANGE
 #include "H5FDhttp.h"
 #endif
 
+#ifdef ENABLE_HDF5_ROS3
+#include <H5FDros3.h>
+#endif
+
 /*Nemonic */
 #define FILTERACTIVE 1
 
@@ -50,11 +58,6 @@ static const int nc_type_size_g[NUM_TYPES] = {sizeof(char), sizeof(char), sizeof
 /** @internal These flags may not be set for open mode. */
 static const int ILLEGAL_OPEN_FLAGS = (NC_MMAP);
 
-/* From libsrc4, these are the netcdf-4 cache sizes. */
-extern size_t nc4_chunk_cache_size;
-extern size_t nc4_chunk_cache_nelems;
-extern float nc4_chunk_cache_preemption;
-
 /* From nc4mem.c */
 extern int NC4_open_image_file(NC_FILE_INFO_T* h5);
 
@@ -69,7 +72,11 @@ typedef struct hdf5_obj_info
 {
     hid_t oid;                          /* HDF5 object ID */
     char oname[NC_MAX_NAME + 1];        /* Name of object */
-    H5G_stat_t statbuf;                 /* Information about the object */
+#if H5_VERSION_GE(1,12,0)
+    H5O_info2_t statbuf;
+#else
+    H5G_stat_t statbuf;                /* Information about the object */
+#endif
     struct hdf5_obj_info *next; /* Pointer to next node in list */
 } hdf5_obj_info_t;
 
@@ -287,7 +294,8 @@ read_coord_dimids(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var)
     /* There is a hidden attribute telling us the ids of the
      * dimensions that apply to this multi-dimensional coordinate
      * variable. Read it. */
-    if ((coord_attid = H5Aopen_name(hdf5_var->hdf_datasetid, COORDINATES)) < 0)
+    if ((coord_attid = H5Aopen_by_name(hdf5_var->hdf_datasetid, ".", COORDINATES, 
+                                       H5P_DEFAULT, H5P_DEFAULT)) < 0)
         BAIL(NC_EATTMETA);
 
     if ((coord_att_typeid = H5Aget_type(coord_attid)) < 0)
@@ -343,11 +351,22 @@ static herr_t
 dimscale_visitor(hid_t did, unsigned dim, hid_t dsid,
                  void *dimscale_hdf5_objids)
 {
-    H5G_stat_t statbuf;
 
     LOG((4, "%s", __func__));
 
     /* Get more info on the dimscale object.*/
+#if H5_VERSION_GE(1,12,0)
+    H5O_info2_t statbuf;
+
+    if (H5Oget_info3(dsid, &statbuf, H5O_INFO_BASIC) < 0)
+        return -1;
+
+    /* Pass this information back to caller. */
+    (*(HDF5_OBJID_T *)dimscale_hdf5_objids).fileno = statbuf.fileno;
+    (*(HDF5_OBJID_T *)dimscale_hdf5_objids).token = statbuf.token;
+#else
+    H5G_stat_t statbuf;
+
     if (H5Gget_objinfo(dsid, ".", 1, &statbuf) < 0)
         return -1;
 
@@ -356,6 +375,7 @@ dimscale_visitor(hid_t did, unsigned dim, hid_t dsid,
     (*(HDF5_OBJID_T *)dimscale_hdf5_objids).fileno[1] = statbuf.fileno[1];
     (*(HDF5_OBJID_T *)dimscale_hdf5_objids).objno[0] = statbuf.objno[0];
     (*(HDF5_OBJID_T *)dimscale_hdf5_objids).objno[1] = statbuf.objno[1];
+#endif
     return 0;
 }
 
@@ -529,7 +549,7 @@ rec_match_dimscales(NC_GRP_INFO_T *grp)
         }
 
         /* Skip dimension scale variables */
-        if (var->dimscale)
+        if (hdf5_var->dimscale)
             continue;
 
         /* If we have already read hidden coordinates att, then we don't
@@ -538,7 +558,7 @@ rec_match_dimscales(NC_GRP_INFO_T *grp)
             continue;
 
         /* Skip dimension scale variables */
-        if (!var->dimscale)
+        if (!hdf5_var->dimscale)
         {
             int d;
             int j;
@@ -573,10 +593,20 @@ rec_match_dimscales(NC_GRP_INFO_T *grp)
 
                             /* Check for exact match of fileno/objid arrays
                              * to find identical objects in HDF5 file. */
+#if H5_VERSION_GE(1,12,0)
+                            int token_cmp;
+                            if (H5Otoken_cmp(hdf5_var->hdf_datasetid,
+                                             &hdf5_var->dimscale_hdf5_objids[d].token,
+                                             &hdf5_dim->hdf5_objid.token, &token_cmp) < 0)
+                                return NC_EHDFERR;
+                            if (hdf5_var->dimscale_hdf5_objids[d].fileno == hdf5_dim->hdf5_objid.fileno &&
+                                token_cmp == 0)
+#else
                             if (hdf5_var->dimscale_hdf5_objids[d].fileno[0] == hdf5_dim->hdf5_objid.fileno[0] &&
                                 hdf5_var->dimscale_hdf5_objids[d].objno[0] == hdf5_dim->hdf5_objid.objno[0] &&
                                 hdf5_var->dimscale_hdf5_objids[d].fileno[1] == hdf5_dim->hdf5_objid.fileno[1] &&
                                 hdf5_var->dimscale_hdf5_objids[d].objno[1] == hdf5_dim->hdf5_objid.objno[1])
+#endif
                             {
                                 LOG((4, "%s: for dimension %d, found dim %s", __func__,
                                      d, dim->hdr.name));
@@ -697,14 +727,17 @@ nc4_open_file(const char *path, int mode, void* parameters, int ncid)
     h5 = (NC_HDF5_FILE_INFO_T*)nc4_info->format_file_info;
 
 #ifdef ENABLE_BYTERANGE
-    /* See if we want the byte range protocol */
-    if(NC_testmode(path,"bytes")) {
-        h5->http.iosp = 1;
-        /* Kill off any conflicting modes flags */
-        mode &= ~(NC_WRITE|NC_DISKLESS|NC_PERSIST|NC_INMEMORY);
-        parameters = NULL; /* kill off parallel */
-    } else
-        h5->http.iosp = 0;
+    /* Do path as URL processing */
+    ncuriparse(path,&h5->uri);
+    if(h5->uri != NULL) {
+        /* See if we want the byte range protocol */
+        if(NC_testmode(h5->uri,"bytes")) h5->byterange = 1; else h5->byterange = 0;
+	if(h5->byterange) {
+  	    /* Kill off any conflicting modes flags */
+	    mode &= ~(NC_WRITE|NC_DISKLESS|NC_PERSIST|NC_INMEMORY);
+            parameters = NULL; /* kill off parallel */
+	}
+    }
 #endif /*ENABLE_BYTERANGE*/
 
     nc4_info->mem.inmemory = ((mode & NC_INMEMORY) == NC_INMEMORY);
@@ -715,39 +748,44 @@ nc4_open_file(const char *path, int mode, void* parameters, int ncid)
     if ((mode & NC_WRITE) == 0)
         nc4_info->no_write = NC_TRUE;
 
+    if ((mode & NC_WRITE) && (mode & NC_NOATTCREORD)) {
+        nc4_info->no_attr_create_order = NC_TRUE;
+    }
+
     if(nc4_info->mem.inmemory && nc4_info->mem.diskless)
         BAIL(NC_EINTERNAL);
 
 #ifdef USE_PARALLEL4
-    mpiinfo = (NC_MPI_INFO*)parameters; /* assume, may be changed if inmemory is true */
+    mpiinfo = (NC_MPI_INFO *)parameters; /* assume, may be changed if inmemory is true */
 #endif /* !USE_PARALLEL4 */
 
-    /* Need this access plist to control how HDF5 handles open objects
-     * on file close. (Setting H5F_CLOSE_SEMI will cause H5Fclose to
-     * fail if there are any open objects in the file). */
+    /* Need this FILE ACCESS plist to control how HDF5 handles open
+     * objects on file close; as well as for other controls below.
+     * (Setting H5F_CLOSE_WEAK will cause H5Fclose not to fail if there
+     * are any open objects in the file. This may happen when virtual
+     * datasets are opened). */
     if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0)
         BAIL(NC_EHDFERR);
 
-    if (H5Pset_fclose_degree(fapl_id, H5F_CLOSE_SEMI) < 0)
+    if (H5Pset_fclose_degree(fapl_id, H5F_CLOSE_WEAK) < 0)
         BAIL(NC_EHDFERR);
 
 #ifdef USE_PARALLEL4
     if (!(mode & (NC_INMEMORY | NC_DISKLESS)) && mpiinfo != NULL) {
         /* If this is a parallel file create, set up the file creation
-         * property list.
-         */
+         * property list. */
         nc4_info->parallel = NC_TRUE;
         LOG((4, "opening parallel file with MPI/IO"));
         if (H5Pset_fapl_mpio(fapl_id, mpiinfo->comm, mpiinfo->info) < 0)
             BAIL(NC_EPARINIT);
 
         /* Keep copies of the MPI Comm & Info objects */
-        if (MPI_SUCCESS != MPI_Comm_dup(mpiinfo->comm, &nc4_info->comm))
+        if (MPI_Comm_dup(mpiinfo->comm, &nc4_info->comm) != MPI_SUCCESS)
             BAIL(NC_EMPI);
         comm_duped++;
-        if (MPI_INFO_NULL != mpiinfo->info)
+        if (mpiinfo->info != MPI_INFO_NULL)
         {
-            if (MPI_SUCCESS != MPI_Info_dup(mpiinfo->info, &nc4_info->info))
+            if (MPI_Info_dup(mpiinfo->info, &nc4_info->info) != MPI_SUCCESS)
                 BAIL(NC_EMPI);
             info_duped++;
         }
@@ -759,19 +797,41 @@ nc4_open_file(const char *path, int mode, void* parameters, int ncid)
     }
 
 #ifdef HDF5_HAS_COLL_METADATA_OPS
+    /* If collective metadata operations are available in HDF5, turn
+     * them on. */
     if (H5Pset_all_coll_metadata_ops(fapl_id, 1) < 0)
         BAIL(NC_EPARINIT);
-#endif
-
-#else /* only set cache for non-parallel. */
-    if (H5Pset_cache(fapl_id, 0, nc4_chunk_cache_nelems, nc4_chunk_cache_size,
-                     nc4_chunk_cache_preemption) < 0)
-        BAIL(NC_EHDFERR);
-    LOG((4, "%s: set HDF raw chunk cache to size %d nelems %d preemption %f",
-         __func__, nc4_chunk_cache_size, nc4_chunk_cache_nelems,
-         nc4_chunk_cache_preemption));
+#endif /* HDF5_HAS_COLL_METADATA_OPS */
 #endif /* USE_PARALLEL4 */
 
+    /* Only set cache for non-parallel opens. */
+    if (!nc4_info->parallel)
+    {
+	NCglobalstate* gs = NC_getglobalstate();
+	if (H5Pset_cache(fapl_id, 0, gs->chunkcache.nelems, gs->chunkcache.size,
+			 gs->chunkcache.preemption) < 0)
+	    BAIL(NC_EHDFERR);
+	LOG((4, "%s: set HDF raw chunk cache to size %d nelems %d preemption %f",
+	     __func__, gs->chunkcache.size, gs->chunkcache.nelems,
+	     gs->chunkcache.preemption));
+    }
+
+    {
+	NCglobalstate* gs = NC_getglobalstate();
+        if(gs->alignment.defined) {
+	    if (H5Pset_alignment(fapl_id, gs->alignment.threshold, gs->alignment.alignment) < 0) {
+	        BAIL(NC_EHDFERR);
+	    }
+	}
+    }
+
+    /* Set HDF5 format compatibility in the FILE ACCESS property list.
+     * Compatibility is transient and must be reselected every time
+     * a file is opened for writing. */
+    retval = hdf5set_format_compatibility(fapl_id);
+    if (retval != NC_NOERR)
+        BAIL(retval);
+
     /* Process  NC_INMEMORY */
     if(nc4_info->mem.inmemory) {
         NC_memio* memio;
@@ -802,26 +862,88 @@ nc4_open_file(const char *path, int mode, void* parameters, int ncid)
             if (H5Pset_fapl_core(fapl_id, min_incr, (nc4_info->mem.persist?1:0)) < 0)
                 BAIL(NC_EHDFERR);
             /* Open the HDF5 file. */
-            if ((h5->hdfid = H5Fopen(path, flags, fapl_id)) < 0)
+            if ((h5->hdfid = nc4_H5Fopen(path, flags, fapl_id)) < 0)
                 BAIL(NC_EHDFERR);
         }
 #ifdef ENABLE_BYTERANGE
-        else
-            if(h5->http.iosp) {   /* Arrange to use the byte-range driver */
-                /* Configure FAPL to use the byte-range file driver */
+	else if(h5->byterange) {   /* Arrange to use the byte-range drivers */
+	    char* newpath = NULL;
+            char* awsregion0 = NULL;
+#ifdef ENABLE_HDF5_ROS3
+	    H5FD_ros3_fapl_t fa;
+	    const char* profile0 = NULL;
+	    const char* awsaccessid0 = NULL;
+	    const char* awssecretkey0 = NULL;
+	    int iss3 = NC_iss3(h5->uri);
+	    
+            fa.version = H5FD_CURR_ROS3_FAPL_T_VERSION;
+	    fa.authenticate = (hbool_t)0;
+	    fa.aws_region[0] = '\0';
+	    fa.secret_id[0] = '\0';
+	    fa.secret_key[0] = '\0';
+
+	    if(iss3) {
+	        /* Rebuild the URL */
+		NCURI* newuri = NULL;
+		if((retval = NC_s3urlrebuild(h5->uri,&newuri,NULL,&awsregion0))) goto exit;
+		if((newpath = ncuribuild(newuri,NULL,NULL,NCURISVC))==NULL)
+		    {retval = NC_EURL; goto exit;}
+		ncurifree(h5->uri);
+		h5->uri = newuri;
+	        if((retval = NC_getactives3profile(h5->uri,&profile0)))
+		    BAIL(retval);
+   	        if((retval = NC_s3profilelookup(profile0,AWS_ACCESS_KEY_ID,&awsaccessid0)))
+		    BAIL(retval);		
+	        if((retval = NC_s3profilelookup(profile0,AWS_SECRET_ACCESS_KEY,&awssecretkey0)))
+		    BAIL(retval);		
+		if(awsregion0 == NULL)
+		    awsregion0 = strdup(S3_REGION_DEFAULT);
+	        if(awsaccessid0 == NULL || awssecretkey0 == NULL ) {
+		    /* default, non-authenticating, "anonymous" fapl configuration */
+		    fa.authenticate = (hbool_t)0;
+	        } else {
+		    fa.authenticate = (hbool_t)1;
+	  	    assert(awsregion0 != NULL && strlen(awsregion0) > 0);
+		    assert(awsaccessid0 != NULL && strlen(awsaccessid0) > 0);
+		    assert(awssecretkey0 != NULL && strlen(awssecretkey0) > 0);
+		    strlcat(fa.aws_region,awsregion0,H5FD_ROS3_MAX_REGION_LEN);
+		    strlcat(fa.secret_id, awsaccessid0, H5FD_ROS3_MAX_SECRET_ID_LEN);
+                    strlcat(fa.secret_key, awssecretkey0, H5FD_ROS3_MAX_SECRET_KEY_LEN);
+	        }
+                /* create and set fapl entry */
+                if(H5Pset_fapl_ros3(fapl_id, &fa) < 0)
+                    BAIL(NC_EHDFERR);
+	    } else
+#endif /*ENABLE_ROS3*/
+	    {/* Configure FAPL to use our byte-range file driver */
                 if (H5Pset_fapl_http(fapl_id) < 0)
                     BAIL(NC_EHDFERR);
-                /* Open the HDF5 file. */
-                if ((h5->hdfid = H5Fopen(path, flags, fapl_id)) < 0)
-                    BAIL(NC_EHDFERR);
-            }
+	    }
+            /* Open the HDF5 file. */
+            if ((h5->hdfid = nc4_H5Fopen((newpath?newpath:path), flags, fapl_id)) < 0)
+                BAIL(NC_EHDFERR);
+	    nullfree(newpath);
+	    nullfree(awsregion0);
+        }
 #endif
-            else
-            {
-                /* Open the HDF5 file. */
-                if ((h5->hdfid = H5Fopen(path, flags, fapl_id)) < 0)
-                    BAIL(NC_EHDFERR);
-            }
+        else {
+            /* Open the HDF5 file. */
+            if ((h5->hdfid = nc4_H5Fopen(path, flags, fapl_id)) < 0)
+                BAIL(NC_EHDFERR);
+        }
+
+    /* Get the file creation property list to check for attribute ordering */
+    {
+      hid_t pid;
+      unsigned int crt_order_flags;
+      if ((pid = H5Fget_create_plist(h5->hdfid)) < 0)
+          BAIL(NC_EHDFERR);
+      if (H5Pget_attr_creation_order(pid, &crt_order_flags) < 0)
+          BAIL(NC_EHDFERR);
+      if (!(crt_order_flags & H5P_CRT_ORDER_TRACKED)) {
+	  nc4_info->no_attr_create_order = NC_TRUE;
+      }
+    }
 
     /* Now read in all the metadata. Some types and dimscale
      * information may be difficult to resolve here, if, for example, a
@@ -867,7 +989,7 @@ exit:
         H5Pclose(fapl_id);
     if (nc4_info)
         nc4_close_hdf5_file(nc4_info, 1, 0); /*  treat like abort*/
-    return retval;
+    return THROW(retval);
 }
 
 /**
@@ -933,87 +1055,81 @@ static int get_filter_info(hid_t propid, NC_VAR_INFO_T *var)
 {
     H5Z_filter_t filter;
     int num_filters;
-    unsigned int cd_values_zip[CD_NELEMS_ZLIB];
-    size_t cd_nelems = CD_NELEMS_ZLIB;
+    unsigned int* cd_values = NULL;
+    size_t cd_nelems;
     int f;
     int stat = NC_NOERR;
+    NC_HDF5_VAR_INFO_T *hdf5_var;
 
     assert(var);
 
+    /* Get HDF5-sepecific var info. */
+    hdf5_var = (NC_HDF5_VAR_INFO_T *)var->format_var_info;
+
     if ((num_filters = H5Pget_nfilters(propid)) < 0)
-        return NC_EHDFERR;
+	{stat = NC_EHDFERR; goto done;}
 
     for (f = 0; f < num_filters; f++)
     {
-        if ((filter = H5Pget_filter2(propid, f, NULL, &cd_nelems, cd_values_zip,
-                                     0, NULL, NULL)) < 0)
-            return NC_EHDFERR;
-        switch (filter)
+	int flags = 0;
+	htri_t avail = -1;
+	cd_nelems = 0;
+        if ((filter = H5Pget_filter2(propid, f, NULL, &cd_nelems, NULL, 0, NULL, NULL)) < 0)
+ 	    {stat = NC_ENOFILTER; goto done;} /* Assume this means an unknown filter */
+	if((avail = H5Zfilter_avail(filter)) < 0)
+ 	    {stat = NC_EHDFERR; goto done;} /* Something in HDF5 went wrong */
+	if(!avail) {
+	    flags |= NC_HDF5_FILTER_MISSING;
+	    /* mark variable as unreadable */
+	    hdf5_var->flags |= NC_HDF5_VAR_FILTER_MISSING;
+	}
+	if((cd_values = calloc(sizeof(unsigned int),cd_nelems))==NULL)
+ 	    {stat = NC_ENOMEM; goto done;}
+        if ((filter = H5Pget_filter2(propid, f, NULL, &cd_nelems, cd_values, 0, NULL, NULL)) < 0)
+ 	    {stat = NC_EHDFERR; goto done;} /* Something in HDF5 went wrong */
+	switch (filter)
         {
-        case H5Z_FILTER_SHUFFLE:
-            var->shuffle = NC_TRUE;
-            break;
-
-        case H5Z_FILTER_FLETCHER32:
-            var->fletcher32 = NC_TRUE;
-            break;
-
         case H5Z_FILTER_DEFLATE:
             if (cd_nelems != CD_NELEMS_ZLIB ||
-                cd_values_zip[0] > NC_MAX_DEFLATE_LEVEL)
-                return NC_EHDFERR;
-	    if((stat = NC4_hdf5_addfilter(var,FILTERACTIVE,filter,cd_nelems,cd_values_zip)))
-		return stat;
+                cd_values[0] > NC_MAX_DEFLATE_LEVEL)
+		    {stat = NC_EHDFERR; goto done;}
+	    if((stat = NC4_hdf5_addfilter(var,filter,cd_nelems,cd_values,flags)))
+	       goto done;
             break;
 
         case H5Z_FILTER_SZIP: {
             /* Szip is tricky because the filter code expands the set of parameters from 2 to 4
                and changes some of the parameter values; try to compensate */
             if(cd_nelems == 0) {
-		if((stat = NC4_hdf5_addfilter(var,FILTERACTIVE,filter,0,NULL)))
-		   return stat;
+		if((stat = NC4_hdf5_addfilter(var,filter,0,NULL,flags)))
+		   goto done;
             } else {
-                /* We have to re-read the parameters based on actual nparams,
-                   which in the case of szip, differs from users original nparams */
-                unsigned int* realparams = (unsigned int*)calloc(1,sizeof(unsigned int)*cd_nelems);
-                if(realparams == NULL)
-                    return NC_ENOMEM;
-                if((filter = H5Pget_filter2(propid, f, NULL, &cd_nelems,
-                                            realparams, 0, NULL, NULL)) < 0) 
-                    return NC_EHDFERR;
                 /* fix up the parameters and the #params */
 		if(cd_nelems != 4)
-		    return NC_EHDFERR;
+		    {stat = NC_EHDFERR; goto done;}
 		cd_nelems = 2; /* ignore last two */		
 		/* Fix up changed params */
-		realparams[0] &= (H5_SZIP_ALL_MASKS);
+		cd_values[0] &= (H5_SZIP_ALL_MASKS);
 		/* Save info */
-		stat = NC4_hdf5_addfilter(var,FILTERACTIVE,filter,cd_nelems,realparams);
-		nullfree(realparams);
-		if(stat) return stat;
-
+		stat = NC4_hdf5_addfilter(var,filter,cd_nelems,cd_values,flags);
+		if(stat) goto done;
             }
             } break;
 
         default:
             if(cd_nelems == 0) {
-  	        if((stat = NC4_hdf5_addfilter(var,FILTERACTIVE,filter,0,NULL))) return stat;
+  	        if((stat = NC4_hdf5_addfilter(var,filter,0,NULL,flags))) goto done;
             } else {
-                /* We have to re-read the parameters based on actual nparams */
-                unsigned int* realparams = (unsigned int*)calloc(1,sizeof(unsigned int)*cd_nelems);
-                if(realparams == NULL)
-                    return NC_ENOMEM;
-                if((filter = H5Pget_filter2(propid, f, NULL, &cd_nelems,
-                                            realparams, 0, NULL, NULL)) < 0)
-                    return NC_EHDFERR;
-  	        stat = NC4_hdf5_addfilter(var,FILTERACTIVE,filter,cd_nelems,realparams);
-		nullfree(realparams);
-		if(stat) return stat;
+  	        stat = NC4_hdf5_addfilter(var,filter,cd_nelems,cd_values,flags);
+		if(stat) goto done;
             }
             break;
         }
+	nullfree(cd_values); cd_values = NULL;
     }
-    return NC_NOERR;
+done:
+    nullfree(cd_values);
+    return stat;
 }
 
 /**
@@ -1042,6 +1158,8 @@ static int get_fill_info(hid_t propid, NC_VAR_INFO_T *var)
         /* Allocate space to hold the fill value. */
         if (!var->fill_value)
         {
+#ifdef SEPDATA
+
             if (var->type_info->nc_type_class == NC_VLEN)
             {
                 if (!(var->fill_value = malloc(sizeof(nc_vlen_t))))
@@ -1053,6 +1171,7 @@ static int get_fill_info(hid_t propid, NC_VAR_INFO_T *var)
                     return NC_ENOMEM;
             }
             else
+#endif
             {
                 assert(var->type_info->size);
                 if (!(var->fill_value = malloc(var->type_info->size)))
@@ -1071,6 +1190,66 @@ static int get_fill_info(hid_t propid, NC_VAR_INFO_T *var)
     return NC_NOERR;
 }
 
+/**
+ * @internal Learn if quantize has been applied to this var. If so,
+ * find the mode and the number of significant digit settings.
+ *
+ * @param var Pointer to NC_VAR_INFO_T for this variable.
+ *
+ * @return ::NC_NOERR No error.
+ * @return ::NC_ENOMEM Out of memory.
+ * @return ::NC_EHDFERR HDF5 returned error.
+ * @author Dennis Heimbigner, Ed Hartnett
+ */
+static int get_quantize_info(NC_VAR_INFO_T *var)
+{
+    hid_t attid;
+    hid_t datasetid;
+
+    /* Try to open an attribute of the correct name for quantize
+     * info. */
+    datasetid = ((NC_HDF5_VAR_INFO_T *)var->format_var_info)->hdf_datasetid;
+    attid = H5Aopen_by_name(datasetid, ".", NC_QUANTIZE_BITGROOM_ATT_NAME,
+			    H5P_DEFAULT, H5P_DEFAULT);
+
+    if (attid > 0)
+      {
+	var->quantize_mode = NC_QUANTIZE_BITGROOM;
+      }
+    else
+      {
+	attid = H5Aopen_by_name(datasetid, ".", NC_QUANTIZE_GRANULARBR_ATT_NAME,
+			    H5P_DEFAULT, H5P_DEFAULT);
+	if (attid > 0)
+	  {
+	    var->quantize_mode = NC_QUANTIZE_GRANULARBR;
+	  }
+	else
+	  {
+	    attid = H5Aopen_by_name(datasetid, ".", NC_QUANTIZE_BITROUND_ATT_NAME,
+				    H5P_DEFAULT, H5P_DEFAULT);
+	    if (attid > 0)
+	      var->quantize_mode = NC_QUANTIZE_BITROUND;
+	  }
+      }
+    
+    /* If there is an attribute, read it for the nsd. */
+    if (attid > 0)
+    {
+        if (H5Aread(attid, H5T_NATIVE_INT, &var->nsd) < 0)
+            return NC_EHDFERR;
+	if (H5Aclose(attid) < 0)
+            return NC_EHDFERR;
+    }
+    else
+    {
+	var->quantize_mode = NC_NOQUANTIZE;
+	var->nsd = 0;
+    }
+
+    return NC_NOERR;
+}
+
 /**
  * @internal Learn the storage and (if chunked) chunksizes of a var.
  *
@@ -1112,6 +1291,16 @@ get_chunking_info(hid_t propid, NC_VAR_INFO_T *var)
     {
 	var->storage = NC_COMPACT;
     }
+#ifdef H5D_VIRTUAL
+    else if (layout == H5D_VIRTUAL)
+    {
+	var->storage = NC_VIRTUAL;
+    }
+#endif
+    else
+    {
+    var->storage = NC_UNKNOWN_STORAGE;
+    }
 
     return NC_NOERR;
 }
@@ -1151,13 +1340,13 @@ get_attached_info(NC_VAR_INFO_T *var, NC_HDF5_VAR_INFO_T *hdf5_var, int ndims,
 
     /* If an enddef has already been called, the dimscales will already
      * be taken care of. */
-    if (num_scales && ndims && !var->dimscale_attached)
+    if (num_scales && ndims && !hdf5_var->dimscale_attached)
     {
         /* Allocate space to remember whether the dimscale has been
          * attached for each dimension, and the HDF5 object IDs of the
          * scale(s). */
         assert(!hdf5_var->dimscale_hdf5_objids);
-        if (!(var->dimscale_attached = calloc(ndims, sizeof(nc_bool_t))))
+        if (!(hdf5_var->dimscale_attached = calloc(ndims, sizeof(nc_bool_t))))
             return NC_ENOMEM;
         if (!(hdf5_var->dimscale_hdf5_objids = malloc(ndims *
                                                       sizeof(struct hdf5_objid))))
@@ -1171,7 +1360,7 @@ get_attached_info(NC_VAR_INFO_T *var, NC_HDF5_VAR_INFO_T *hdf5_var, int ndims,
             if (H5DSiterate_scales(hdf5_var->hdf_datasetid, d, NULL, dimscale_visitor,
                                    &(hdf5_var->dimscale_hdf5_objids[d])) < 0)
                 return NC_EHDFERR;
-            var->dimscale_attached[d] = NC_TRUE;
+            hdf5_var->dimscale_attached[d] = NC_TRUE;
             LOG((4, "dimscale attached"));
         }
     }
@@ -1208,7 +1397,7 @@ get_scale_info(NC_GRP_INFO_T *grp, NC_DIM_INFO_T *dim, NC_VAR_INFO_T *var,
     if (dim)
     {
         assert(ndims);
-        var->dimscale = NC_TRUE;
+        hdf5_var->dimscale = NC_TRUE;
 
         /* If this is a multi-dimensional coordinate var, then the
          * dimids must be stored in the hidden coordinates attribute. */
@@ -1272,10 +1461,10 @@ nc4_get_var_meta(NC_VAR_INFO_T *var)
         BAIL(NC_EVARMETA);
 
     /* Learn about current chunk cache settings. */
-    if ((H5Pget_chunk_cache(access_pid, &(var->chunk_cache_nelems),
-                            &(var->chunk_cache_size), &rdcc_w0)) < 0)
+    if ((H5Pget_chunk_cache(access_pid, &(var->chunkcache.nelems),
+                            &(var->chunkcache.size), &rdcc_w0)) < 0)
         BAIL(NC_EHDFERR);
-    var->chunk_cache_preemption = rdcc_w0;
+    var->chunkcache.preemption = rdcc_w0;
 
     /* Get the dataset creation properties. */
     if ((propid = H5Dget_create_plist(hdf5_var->hdf_datasetid)) < 0)
@@ -1298,7 +1487,11 @@ nc4_get_var_meta(NC_VAR_INFO_T *var)
     if ((retval = nc4_adjust_var_cache(var->container, var)))
         BAIL(retval);
 
-    if (var->coords_read && !var->dimscale)
+    /* Is there an attribute which means quantization was used? */
+    if ((retval = get_quantize_info(var)))
+	BAIL(retval);
+
+    if (var->coords_read && !hdf5_var->dimscale)
         if ((retval = get_attached_info(var, hdf5_var, var->ndims, hdf5_var->hdf_datasetid)))
             return retval;
 
@@ -1377,6 +1570,9 @@ read_var(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
     var->created = NC_TRUE;
     var->atts_read = 0;
 
+    /* Create filter list */
+    var->filters = (void*)nclistnew();
+
     /* Try and read the dimids from the COORDINATES attribute. If it's
      * not present, we will have to do dimsscale matching to locate the
      * dims for this var. */
@@ -1399,6 +1595,9 @@ read_var(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
     /* Indicate that the variable has a pointer to the type */
     var->type_info->rc++;
 
+    /* Transfer endianness */
+    var->endianness = var->type_info->endianness; 
+
 exit:
     if (finalname)
         free(finalname);
@@ -1408,6 +1607,10 @@ exit:
          * delete the var info struct we just created. */
         if (incr_id_rc && H5Idec_ref(datasetid) < 0)
             BAIL2(NC_EHDFERR);
+	if(var && var->format_var_info)
+	    free(var->format_var_info);
+	if(var && var->filters)
+	    nclistfree(var->filters);
         if (var)
             nc4_var_list_del(grp, var);
     }
@@ -1573,7 +1776,7 @@ read_hdf5_att(NC_GRP_INFO_T *grp, hid_t attid, NC_ATT_INFO_T *att)
     LOG((5, "%s: att->hdr.id %d att->hdr.name %s att->nc_typeid %d att->len %d",
          __func__, att->hdr.id, att->hdr.name, (int)att->nc_typeid, att->len));
 
-    /* Get HDF5-sepecific info struct for this attribute. */
+    /* Get HDF5-specific info struct for this attribute. */
     hdf5_att = (NC_HDF5_ATT_INFO_T *)att->format_att_info;
 
     /* Get type of attribute in file. */
@@ -1595,7 +1798,6 @@ read_hdf5_att(NC_GRP_INFO_T *grp, hid_t attid, NC_ATT_INFO_T *att)
                                   &(att->nc_typeid))))
         BAIL(retval);
 
-
     /* Get len. */
     if ((spaceid = H5Aget_space(attid)) < 0)
         BAIL(NC_EATTMETA);
@@ -1664,6 +1866,7 @@ read_hdf5_att(NC_GRP_INFO_T *grp, hid_t attid, NC_ATT_INFO_T *att)
         if ((retval = nc4_get_typelen_mem(grp->nc4_info, att->nc_typeid,
                                           &type_size)))
             return retval;
+#ifdef SEPDATA
         if (att_class == H5T_VLEN)
         {
             if (!(att->vldata = malloc((unsigned int)(att->len * sizeof(hvl_t)))))
@@ -1680,11 +1883,11 @@ read_hdf5_att(NC_GRP_INFO_T *grp, hid_t attid, NC_ATT_INFO_T *att)
              * nc_free_string be called on string arrays, which would not
              * work if one contiguous memory block were used. So here I
              * convert the contiguous block of strings into an array of
-             * malloced strings (each string with its own malloc). Then I
+             * malloced strings -- each string with its own malloc. Then I
              * copy the data and free the contiguous memory. This
              * involves copying the data, which is bad, but this only
              * occurs for fixed length string attributes, and presumably
-             * these are small. (And netCDF-4 does not create them - it
+             * these are small. Note also that netCDF-4 does not create them - it
              * always uses variable length strings. */
             if (fixed_len_string)
             {
@@ -1726,12 +1929,64 @@ read_hdf5_att(NC_GRP_INFO_T *grp, hid_t attid, NC_ATT_INFO_T *att)
             }
         }
         else
+#else
         {
             if (!(att->data = malloc((unsigned int)(att->len * type_size))))
                 BAIL(NC_ENOMEM);
-            if (H5Aread(attid, hdf5_att->native_hdf_typeid, att->data) < 0)
-                BAIL(NC_EATTMETA);
+
+            /* For a fixed length HDF5 string, the read requires
+             * contiguous memory. Meanwhile, the netCDF API requires that
+             * nc_free_string be called on string arrays, which would not
+             * work if one contiguous memory block were used. So here I
+             * convert the contiguous block of strings into an array of
+             * malloced strings -- each string with its own malloc. Then I
+             * copy the data and free the contiguous memory. This
+             * involves copying the data, which is bad, but this only
+             * occurs for fixed length string attributes, and presumably
+             * these are small. Note also that netCDF-4 does not create them - it
+             * always uses variable length strings. */
+            if (att->nc_typeid == NC_STRING && fixed_len_string)
+            {
+                int i;
+                char *contig_buf, *cur;
+		char** dst = NULL;
+
+                /* Alloc space for the contiguous memory read. */
+                if (!(contig_buf = malloc(att->len * fixed_size * sizeof(char))))
+                    BAIL(NC_ENOMEM);
+
+                /* Read the fixed-len strings as one big block. */
+                if (H5Aread(attid, hdf5_att->native_hdf_typeid, contig_buf) < 0) {
+                    free(contig_buf);
+                    BAIL(NC_EATTMETA);
+                }
+
+                /* Copy strings, one at a time, into their new home. Alloc
+                   space for each string. The user will later free this
+                   space with nc_free_string. */
+                cur = contig_buf;
+	        dst = (char**)att->data;
+                for (i = 0; i < att->len; i++)
+                {
+		    char* s = NULL;
+                    if (!(s = malloc(fixed_size+1))) {
+                        free(contig_buf);
+                        BAIL(NC_ENOMEM);
+                    }
+		    memcpy(s,cur,fixed_size);
+		    s[fixed_size] = '\0';
+		    dst[i] = s;
+                    cur += fixed_size;
+                }
+                /* Free contiguous memory buffer. */
+                free(contig_buf);
+            } else { /* not fixed string */
+		/* Just read the data */
+                if (H5Aread(attid, hdf5_att->native_hdf_typeid, att->data) < 0)
+                    BAIL(NC_EATTMETA);
+	    }
         }
+#endif
     }
 
     if (H5Tclose(file_typeid) < 0)
@@ -1763,13 +2018,7 @@ hdf5free(void* memory)
 #ifndef JNA
     /* On Windows using the microsoft runtime, it is an error
        for one library to free memory allocated by a different library.*/
-#ifdef HAVE_H5FREE_MEMORY
     if(memory != NULL) H5free_memory(memory);
-#else
-#ifndef _MSC_VER
-    if(memory != NULL) free(memory);
-#endif
-#endif
 #endif
 }
 
@@ -1904,7 +2153,7 @@ read_type(NC_GRP_INFO_T *grp, hid_t hdf_typeid, char *type_name)
                 if ((ndims = H5Tget_array_ndims(member_hdf_typeid)) < 0)
                     return NC_EHDFERR;
 
-                if (H5Tget_array_dims(member_hdf_typeid, dims, NULL) != ndims)
+                if (H5Tget_array_dims1(member_hdf_typeid, dims, NULL) != ndims)
                     return NC_EHDFERR;
 
                 for (d = 0; d < ndims; d++)
@@ -1933,6 +2182,12 @@ read_type(NC_GRP_INFO_T *grp, hid_t hdf_typeid, char *type_name)
                     return retval;
             }
 
+	    {   /* See if this changes from fixed size to variable size */
+		int fixedsize;
+                if((retval = NC4_inq_type_fixed_size(grp->nc4_info->controller->ext_ncid,member_xtype,&fixedsize))) return retval;
+		if(!fixedsize) type->u.c.varsized = 1;
+	    }
+
             hdf5free(member_name);
         }
     }
@@ -2064,7 +2319,7 @@ read_type(NC_GRP_INFO_T *grp, hid_t hdf_typeid, char *type_name)
  * for both global and variable attributes.
  *
  * @param loc_id HDF5 attribute ID.
- * @param att_name Name of the attrigute.
+ * @param att_name Name of the attribute.
  * @param ainfo HDF5 info struct for attribute.
  * @param att_data Pointer to an att_iter_info struct, which contains
  * pointers to the NC_GRP_INFO_T and (for variable attributes) the
@@ -2108,6 +2363,10 @@ att_read_callbk(hid_t loc_id, const char *att_name, const H5A_info_t *ainfo,
     if ((retval = nc4_att_list_add(list, att_name, &att)))
         BAIL(-1);
 
+    /* Remember container */
+    att->container = (att_info->var ? (NC_OBJ*)att_info->var: (NC_OBJ*)att_info->grp);
+
+
     /* Allocate storage for the HDF5 specific att info. */
     if (!(att->format_att_info = calloc(1, sizeof(NC_HDF5_ATT_INFO_T))))
         BAIL(-1);
@@ -2130,6 +2389,8 @@ exit:
     {
         /* NC_EBADTYPID will be normally converted to NC_NOERR so that
            the parent iterator does not fail. */
+	/* Free up the format_att_info */
+        if((retval=nc4_HDF5_close_att(att))) return retval;
         retval = nc4_att_list_del(list, att);
         att = NULL;
     }
@@ -2204,10 +2465,16 @@ nc4_read_atts(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var)
  * @returns ::NC_NOERR No error.
  * @return ::NC_EHDFERR HDF5 returned error.
  * @author Ed Hartnett
+ * [Candidate for libsrc4]
  */
 static int
 read_scale(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
-           const H5G_stat_t *statbuf, hsize_t scale_size,
+#if H5_VERSION_GE(1,12,0)
+           const H5O_info2_t *statbuf,
+#else
+           const H5G_stat_t *statbuf,
+#endif
+           hsize_t scale_size,
            hsize_t max_scale_size, NC_DIM_INFO_T **dim)
 {
     NC_DIM_INFO_T *new_dim; /* Dimension added to group */
@@ -2230,7 +2497,8 @@ read_scale(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
         BAIL(NC_EHDFERR);
     if (attr_exists)
     {
-        if ((attid = H5Aopen_name(datasetid, NC_DIMID_ATT_NAME)) < 0)
+        if ((attid = H5Aopen_by_name(datasetid,".", NC_DIMID_ATT_NAME,
+                                     H5P_DEFAULT, H5P_DEFAULT)) < 0)
             BAIL(NC_EHDFERR);
 
         if (H5Aread(attid, H5T_NATIVE_INT, &assigned_id) < 0)
@@ -2263,12 +2531,17 @@ read_scale(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
 
     dimscale_created++;
 
-    /* Remember these 4 values to uniquely identify this dataset in the
+    /* Remember these 4 (or 2 for HDF5 1.12) values to uniquely identify this dataset in the
      * HDF5 file. */
+#if H5_VERSION_GE(1,12,0)
+    new_hdf5_dim->hdf5_objid.fileno = statbuf->fileno;
+    new_hdf5_dim->hdf5_objid.token = statbuf->token;
+#else
     new_hdf5_dim->hdf5_objid.fileno[0] = statbuf->fileno[0];
     new_hdf5_dim->hdf5_objid.fileno[1] = statbuf->fileno[1];
     new_hdf5_dim->hdf5_objid.objno[0] = statbuf->objno[0];
     new_hdf5_dim->hdf5_objid.objno[1] = statbuf->objno[1];
+#endif
 
     /* If the dimscale has an unlimited dimension, then this dimension
      * is unlimited. */
@@ -2339,7 +2612,12 @@ exit:
  */
 static int
 read_dataset(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
-             const H5G_stat_t *statbuf)
+#if H5_VERSION_GE(1,12,0)
+             const H5O_info2_t *statbuf
+#else
+             const H5G_stat_t *statbuf
+#endif
+)
 {
     NC_DIM_INFO_T *dim = NULL;   /* Dimension created for scales */
     NC_HDF5_DIM_INFO_T *hdf5_dim;
@@ -2438,7 +2716,12 @@ oinfo_list_add(user_data_t *udata, const hdf5_obj_info_t *oinfo)
  * @author Ed Hartnett
  */
 static int
-read_hdf5_obj(hid_t grpid, const char *name, const H5L_info_t *info,
+read_hdf5_obj(hid_t grpid, const char *name,
+#if H5_VERSION_GE(1,12,0)
+	      const H5L_info2_t *info,
+#else
+	      const H5L_info_t *info,
+#endif
               void *_op_data)
 {
     /* Pointer to user data for callback */
@@ -2451,8 +2734,13 @@ read_hdf5_obj(hid_t grpid, const char *name, const H5L_info_t *info,
         BAIL(H5_ITER_ERROR);
 
     /* Get info about the object.*/
+#if H5_VERSION_GE(1,12,0)
+    if (H5Oget_info3(oinfo.oid, &oinfo.statbuf, H5O_INFO_BASIC) < 0)
+        BAIL(H5_ITER_ERROR);
+#else
     if (H5Gget_objinfo(oinfo.oid, ".", 1, &oinfo.statbuf) < 0)
         BAIL(H5_ITER_ERROR);
+#endif
 
     strncpy(oinfo.oname, name, NC_MAX_NAME);
 
@@ -2657,3 +2945,34 @@ exit:
 
     return retval;
 }
+
+/**
+ * Wrapper function for H5Fopen.
+ * Converts the filename from ANSI to UTF-8 as needed before calling H5Fopen.
+ *
+ * @param filename The filename encoded ANSI to access.
+ * @param flags File access flags.
+ * @param fapl_id File access property list identifier.
+ * @return A file identifier if succeeded. A negative value if failed.
+ */
+hid_t
+nc4_H5Fopen(const char *filename0, unsigned flags, hid_t fapl_id)
+{
+    hid_t hid;
+    char* localname = NULL;
+    char* filename = NULL;
+
+#ifdef HDF5_UTF8_PATHS
+    NCpath2utf8(filename0,&filename);
+#else    
+    filename = strdup(filename0);
+#endif
+    if((localname = NCpathcvt(filename))==NULL)
+	{hid = H5I_INVALID_HID; goto done;}
+    hid = H5Fopen(localname, flags, fapl_id);
+
+done:
+    nullfree(filename);
+    nullfree(localname);
+    return hid;
+}
