@@ -17,20 +17,59 @@
 /**************************************************/
 /* Static zarr type name table */
 
-static const char* znames_little[NUM_ATOMIC_TYPES] = {
-NULL,  /*NC_NAT*/
-"<i1", /*NC_BYTE*/ "<U1", /*NC_CHAR*/ "<i2", /*NC_SHORT*/
-"<i4", /*NC_INT*/ "<f4", /*NC_FLOAT*/ "<f8", /*NC_DOUBLE*/ "<u1", /*NC_UBYTE*/
-"<u2", /*NC_USHORT*/ "<u4", /*NC_UINT*/ "<i8", /*NC_INT64*/ "<u8", /*NC_UINT64*/
-NULL,  /*NC_STRING*/
-};
+/* Table of nc_type X {Zarr,NCZarr} X endianness
+Issue: Need to distinquish NC_STRING && MAXSTRLEN==1 from NC_CHAR
+in a way that allows other Zarr implementations to read the data.
 
-static const char* znames_big[NUM_ATOMIC_TYPES] = {
-NULL,  /*NC_NAT*/
-">i1", /*NC_BYTE*/   ">U1", /*NC_CHAR*/   ">i2", /*NC_SHORT*/ ">i4", /*NC_INT*/
-">f4", /*NC_FLOAT*/  ">f8", /*NC_DOUBLE*/ ">u1", /*NC_UBYTE*/
-">u2", /*NC_USHORT*/ ">u4", /*NC_UINT*/   ">i8", /*NC_INT64*/ ">u8", /*NC_UINT64*/
-NULL,  /*NC_STRING*/
+Available info:
+Write: we have the netcdf type, so there is no ambiguity.
+Read: we have the variable type and also any attribute dtype,
+but those types are ambiguous.
+We also have the attribute vs variable type problem.
+For pure zarr, we have to infer the type of an attribute,
+so if we have "var:strattr = \"abcdef\"", then we need
+to decide how to infer the type: NC_STRING vs NC_CHAR.
+
+Solution:
+For variables and for NCZarr type attributes, distinquish by using:
+* ">S1" for NC_CHAR.
+* "|S1" for NC_STRING && MAXSTRLEN==1
+* "|Sn" for NC_STRING && MAXSTRLEN==n
+This is admittedly a bit of a hack, and the first case in particular
+will probably cause errors in some other Zarr implementations; the Zarr
+spec is unclear about what combinations are legal.
+Note that we could use "|U1", but since this is utf-16 or utf-32
+in python, it may cause problems when reading what amounts to utf-8.
+
+For attributes, we infer:
+* NC_CHAR if the hint is 0
+  - e.g. var:strattr = 'abcdef'" => NC_CHAR
+* NC_STRING if hint is NC_STRING.
+  - e.g. string var:strattr = \"abc\", \"def\"" => NC_STRING
+
+Note also that if we read a pure zarr file we will probably always
+see "|S1", so we will never see a variable of type NC_CHAR.
+We might however see an attribute of type string.
+*/
+static const struct ZTYPES {
+    char* zarr[3];
+    char* nczarr[3];
+} znames[NUM_ATOMIC_TYPES] = {
+/* nc_type          Pure Zarr          NCZarr
+                   NE   LE     BE       NE    LE    BE*/
+/*NC_NAT*/	{{NULL,NULL,NULL},   {NULL,NULL,NULL}},
+/*NC_BYTE*/	{{"|i1","<i1",">i1"},{"|i1","<i1",">i1"}},
+/*NC_CHAR*/	{{">S1",">S1",">S1"},{">S1",">S1",">S1"}},
+/*NC_SHORT*/	{{"|i2","<i2",">i2"},{"|i2","<i2",">i2"}},
+/*NC_INT*/	{{"|i4","<i4",">i4"},{"|i4","<i4",">i4"}},
+/*NC_FLOAT*/	{{"|f4","<f4",">f4"},{"|f4","<f4",">f4"}},
+/*NC_DOUBLE*/	{{"|f8","<f8",">f8"},{"|f8","<f8",">f8"}},
+/*NC_UBYTE*/	{{"|u1","<u1",">u1"},{"|u1","<u1",">u1"}},
+/*NC_USHORT*/	{{"|u2","<u2",">u2"},{"|u2","<u2",">u2"}},
+/*NC_UINT*/	{{"|u4","<u4",">u4"},{"|u4","<u4",">u4"}},
+/*NC_INT64*/	{{"|i8","<i8",">i8"},{"|i8","<i8",">i8"}},
+/*NC_UINT64*/	{{"|u8","<u8",">u8"},{"|u8","<u8",">u8"}},
+/*NC_STRING*/	{{"|S%d","|S%d","|S%d"},{"|S%d","|S%d","|S%d"}},
 };
 
 #if 0
@@ -47,6 +86,7 @@ NULL, /*NC_NAT*/
 "4294967295", /*NC_UINT*/
 "-9223372036854775806", /*NC_INT64*/
 "18446744073709551614", /*NC_UINT64*/
+"", /*NC_STRING*/
 };
 #endif
 
@@ -64,10 +104,10 @@ NCJ_INT, /*NC_USHORT*/
 NCJ_INT, /*NC_UINT*/
 NCJ_INT, /*NC_INT64*/
 NCJ_INT, /*NC_UINT64*/
+NCJ_STRING, /*NC_STRING*/
 };
 
 /* Forward */
-static int endswith(const char* s, const char* suffix);
 
 /**************************************************/
 
@@ -205,8 +245,8 @@ NCZ_downloadjson(NCZMAP* zmap, const char* key, NCjson** jsonp)
 	goto done;
     content[len] = '\0';
 
-    if((stat = NCJparse(content,0,&json)))
-	goto done;
+    if((stat = NCJparse(content,0,&json)) < 0)
+	{stat = NC_ENCZARR; goto done;}
 
     if(jsonp) {*jsonp = json; json = NULL;}
 
@@ -340,7 +380,7 @@ NCZ_readdict(NCZMAP* zmap, const char* key, NCjson** jsonp)
 
     if((stat = NCZ_downloadjson(zmap,key,&json)))
 	goto done;
-    if(json->sort != NCJ_DICT) {stat = NC_ENCZARR; goto done;}
+    if(NCJsort(json) != NCJ_DICT) {stat = NC_ENCZARR; goto done;}
     if(jsonp) {*jsonp = json; json = NULL;}
 done:
     NCJreclaim(json);
@@ -364,35 +404,13 @@ NCZ_readarray(NCZMAP* zmap, const char* key, NCjson** jsonp)
 
     if((stat = NCZ_downloadjson(zmap,key,&json)))
 	goto done;
-    if(json->sort != NCJ_ARRAY) {stat = NC_ENCZARR; goto done;}
+    if(NCJsort(json) != NCJ_ARRAY) {stat = NC_ENCZARR; goto done;}
     if(jsonp) {*jsonp = json; json = NULL;}
 done:
     NCJreclaim(json);
     return stat;
 }
 
-/**
-@internal Given an nc_type+other, produce the corresponding
-zarr type name.
-@param nctype - [in] nc_type
-@param little - [in] 1=>little, 0 => big
-@param namep - [out] pointer to hold pointer to the name
-@return NC_NOERR
-@author Dennis Heimbigner
-*/
-
-int
-ncz_zarr_type_name(nc_type nctype, int little, const char** znamep)
-{
-    if(nctype <= 0 || nctype >= NC_STRING) return NC_EINVAL;
-    if(little) {
-	if(znamep) *znamep = znames_little[nctype];
-    } else {
-	if(znamep) *znamep = znames_big[nctype];
-    }
-    return NC_NOERR;	        
-}
-
 #if 0
 /**
 @internal Given an nc_type, produce the corresponding
@@ -406,7 +424,7 @@ default fill value as a string.
 int
 ncz_default_fill_value(nc_type nctype, const char** dfaltp)
 {
-    if(nctype <= 0 || nctype >= NC_STRING) return NC_EINVAL;
+    if(nctype <= 0 || nctype > NC_MAX_ATOMIC_TYPE) return NC_EINVAL;
     if(dfaltp) *dfaltp = zfillvalue[nctype];
     return NC_NOERR;	        
 }
@@ -414,9 +432,9 @@ ncz_default_fill_value(nc_type nctype, const char** dfaltp)
 
 /**
 @internal Given an nc_type, produce the corresponding
-fill value sort
+fill value JSON type
 @param nctype - [in] nc_type
-@param sortp - [out] pointer to hold pointer to the sort
+@param sortp - [out] pointer to hold pointer to the JSON type
 @return NC_NOERR
 @author Dennis Heimbigner
 */
@@ -424,7 +442,7 @@ fill value sort
 int
 ncz_fill_value_sort(nc_type nctype, int* sortp)
 {
-    if(nctype <= 0 || nctype >= NC_STRING) return NC_EINVAL;
+    if(nctype <= 0 || nctype > NC_MAX_ATOMIC_TYPE) return NC_EINVAL;
     if(sortp) *sortp = zjsonsort[nctype];
     return NC_NOERR;	        
 }
@@ -487,57 +505,119 @@ done:
     return stat;
 }
 
+#if 0
+/* Convert a netcdf-4 type integer */
 int
-ncz_nctype2typeinfo(const char* snctype, nc_type* nctypep)
+ncz_nctypedecode(const char* snctype, nc_type* nctypep)
 {
     unsigned nctype = 0;
     if(sscanf(snctype,"%u",&nctype)!=1) return NC_EINVAL;
     if(nctypep) *nctypep = nctype;
     return NC_NOERR;
 }
+#endif
+
+/**
+@internal Given an nc_type+other, produce the corresponding dtype string.
+@param nctype     - [in] nc_type
+@param endianness - [in] endianness
+@param purezarr   - [in] 1=>pure zarr, 0 => nczarr
+@param strlen     - [in] max string length
+@param namep      - [out] pointer to hold pointer to the dtype; user frees
+@return NC_NOERR
+@return NC_EINVAL
+@author Dennis Heimbigner
+*/
 
 int
-ncz_dtype2typeinfo(const char* dtype, nc_type* nctypep, int* endianp)
+ncz_nctype2dtype(nc_type nctype, int endianness, int purezarr, int len, char** dnamep)
+{
+    char dname[64];
+    char* format = NULL;
+
+    if(nctype <= NC_NAT || nctype > NC_MAX_ATOMIC_TYPE) return NC_EINVAL;
+    if(purezarr)
+        format = znames[nctype].zarr[endianness];
+    else
+        format = znames[nctype].nczarr[endianness];
+    snprintf(dname,sizeof(dname),format,len);
+    if(dnamep) *dnamep = strdup(dname);
+    return NC_NOERR;		
+}
+
+/*
+@internal Convert a numcodecs dtype spec to a corresponding nc_type.
+@param nctype   - [in] dtype the dtype to convert
+@param nctype   - [in] typehint help disambiguate char vs string
+@param purezarr - [in] 1=>pure zarr, 0 => nczarr
+@param nctypep  - [out] hold corresponding type
+@param endianp  - [out] hold corresponding endianness
+@param typelenp - [out] hold corresponding type size (for fixed length strings)
+@return NC_NOERR
+@return NC_EINVAL
+@author Dennis Heimbigner
+*/
+
+int
+ncz_dtype2nctype(const char* dtype, nc_type typehint, int purezarr, nc_type* nctypep, int* endianp, int* typelenp)
 {
     int stat = NC_NOERR;
     int typelen = 0;
     int count;
     char tchar;
     nc_type nctype = NC_NAT;
-    int endianness = 0;
+    int endianness = -1;
+    const char* p;
+    int n;
 
-    if(endianness) *endianp = NC_ENDIAN_NATIVE;
+    if(endianp) *endianp = NC_ENDIAN_NATIVE;
     if(nctypep) *nctypep = NC_NAT;
 
     if(dtype == NULL) goto zerr;
-    if(strlen(dtype) < 3) goto zerr;
-    switch (dtype[0]) {
+    p = dtype;
+    switch (*p++) {
     case '<': endianness = NC_ENDIAN_LITTLE; break;
     case '>': endianness = NC_ENDIAN_BIG; break;
     case '|': endianness = NC_ENDIAN_NATIVE; break;
-    default: goto zerr;
+    default: p--; endianness = NC_ENDIAN_NATIVE; break;
     }
+    tchar = *p++; /* get the base type */
     /* Decode the type length */
-    count = sscanf(dtype+2,"%d",&typelen);
-    if(count != 1) goto zerr;
-    tchar = dtype[1];
-    switch(typelen) {
-    case 1:
-	switch (tchar) {
-	case 'i': nctype = NC_BYTE; break;
-	case 'u': nctype = NC_UBYTE; break;
-	case 'U': nctype = NC_CHAR; break;
-	default: goto zerr;
+    count = sscanf(p,"%d%n",&typelen,&n);
+    if(count == 0) goto zerr;
+    p += n;
+
+    /* Short circuit fixed length strings */
+    if(tchar == 'S') {
+	/* Fixed length string */
+	switch (typelen) {
+	case 1:
+	    nctype = (endianness == NC_ENDIAN_BIG ? NC_CHAR : NC_STRING);
+	    if(purezarr) nctype = NC_STRING; /* Zarr has no NC_CHAR type */
+	    break;
+	default:
+	    nctype = NC_STRING;
+	    break;
 	}
-	break;
-    case 2:
+	/* String/char have no endianness */
+	endianness = NC_ENDIAN_NATIVE;
+    } else {
+	switch(typelen) {
+        case 1:
+	    switch (tchar) {
+  	    case 'i': nctype = NC_BYTE; break;
+   	    case 'u': nctype = NC_UBYTE; break;
+	    default: goto zerr;
+	    }
+	    break;
+        case 2:
 	switch (tchar) {
 	case 'i': nctype = NC_SHORT; break;
 	case 'u': nctype = NC_USHORT; break;
 	default: goto zerr;
 	}
 	break;
-    case 4:
+        case 4:
 	switch (tchar) {
 	case 'i': nctype = NC_INT; break;
 	case 'u': nctype = NC_UINT; break;
@@ -545,7 +625,7 @@ ncz_dtype2typeinfo(const char* dtype, nc_type* nctypep, int* endianp)
 	default: goto zerr;
 	}
 	break;
-    case 8:
+        case 8:
 	switch (tchar) {
 	case 'i': nctype = NC_INT64; break;
 	case 'u': nctype = NC_UINT64; break;
@@ -553,10 +633,18 @@ ncz_dtype2typeinfo(const char* dtype, nc_type* nctypep, int* endianp)
 	default: goto zerr;
 	}
 	break;
-    default: goto zerr;
+        default: goto zerr;
+        }
     }
 
+#if 0
+    /* Convert NC_ENDIAN_NATIVE and NC_ENDIAN_NA */
+    if(endianness == NC_ENDIAN_NATIVE)
+        endianness = (NC_isLittleEndian()?NC_ENDIAN_LITTLE:NC_ENDIAN_BIG);
+#endif
+
     if(nctypep) *nctypep = nctype;
+    if(typelenp) *typelenp = typelen;
     if(endianp) *endianp = endianness;
 
 done:
@@ -566,7 +654,104 @@ zerr:
     goto done;
 }
 
+/* Infer the attribute's type based
+primarily on the first atomic value encountered
+recursively.
+*/
+int
+NCZ_inferattrtype(NCjson* value, nc_type typehint, nc_type* typeidp)
+{
+    int i,stat = NC_NOERR;
+    nc_type typeid;
+    NCjson* j = NULL;
+    unsigned long long u64;
+    long long i64;
+    int negative = 0;
 
+    if(NCJsort(value) == NCJ_ARRAY && NCJlength(value) == 0)
+        {typeid = NC_NAT; goto done;} /* Empty array is illegal */
+
+    if(NCJsort(value) == NCJ_NULL)
+        {typeid = NC_NAT; goto done;} /* NULL is also illegal */
+
+    if(NCJsort(value) == NCJ_DICT) /* Complex JSON expr -- a dictionary */
+        {typeid = NC_NAT; goto done;}
+
+    /* If an array, make sure all the elements are simple */
+    if(value->sort == NCJ_ARRAY) {
+	for(i=0;i<NCJlength(value);i++) {
+	    j=NCJith(value,i);
+	    if(!NCJisatomic(j))
+	        {typeid = NC_NAT; goto done;}
+	}
+    }
+
+    /* Infer from first element */
+    if(value->sort == NCJ_ARRAY) {
+        j=NCJith(value,0);
+	return NCZ_inferattrtype(j,typehint,typeidp);
+    }
+
+    /* At this point, value is a primitive JSON Value */
+
+    switch (NCJsort(value)) {
+    case NCJ_NULL:
+        typeid = NC_NAT;
+	return NC_NOERR;
+    case NCJ_DICT:
+    	typeid = NC_CHAR;
+	goto done;
+    case NCJ_UNDEF:
+	return NC_EINVAL;
+    default: /* atomic */
+	break;
+    }
+
+    if(NCJstring(value) != NULL)
+        negative = (NCJstring(value)[0] == '-');
+    switch (value->sort) {
+    case NCJ_INT:
+	if(negative) {
+	    sscanf(NCJstring(value),"%lld",&i64);
+	    u64 = (unsigned long long)i64;
+	} else
+	    sscanf(NCJstring(value),"%llu",&u64);
+	typeid = NCZ_inferinttype(u64,negative);
+	break;
+    case NCJ_DOUBLE:
+	typeid = NC_DOUBLE;
+	break;
+    case NCJ_BOOLEAN:
+	typeid = NC_UBYTE;
+	break;
+    case NCJ_STRING: /* requires special handling as an array of characters */
+	typeid = NC_CHAR;
+	break;
+    default:
+	stat = NC_ENCZARR;
+    }
+done:
+    if(typeidp) *typeidp = typeid;
+    return stat;
+}
+
+/* Infer the int type from the value;
+   minimum type will be int.
+*/
+int
+NCZ_inferinttype(unsigned long long u64, int negative)
+{
+    long long i64 = (long long)u64; /* keep bit pattern */
+    if(!negative && u64 >= NC_MAX_INT64) return NC_UINT64;
+    if(i64 < 0) {
+	if(i64 >= NC_MIN_INT) return NC_INT;
+	return NC_INT64;
+    }
+    if(i64 <= NC_MAX_INT) return NC_INT;
+    if(i64 <= NC_MAX_UINT) return NC_UINT;
+    return NC_INT64;
+}
+ 
 /**
 @internal Similar to NCZ_grppath, but using group ids.
 @param gid - [in] group id
@@ -730,162 +915,6 @@ NCZ_freestringvec(size_t len, char** vec)
     nullfree(vec);
 }
 
-/* create a fill chunk */
-int
-NCZ_create_fill_chunk(size64_t chunksize, size_t typesize, void* fill, void** fillchunkp)
-{
-    int i;
-    void* fillchunk = NULL;
-    if((fillchunk = malloc(chunksize))==NULL)
-        return NC_ENOMEM;
-    switch (typesize) {
-    case 1: {
-        unsigned char c = *((unsigned char*)fill);
-        memset(fillchunk,c,chunksize);
-        } break;
-    case 2: {
-        unsigned short fv = *((unsigned short*)fill);
-        unsigned short* p2 = (unsigned short*)fillchunk;
-        for(i=0;i<chunksize;i+=typesize) *p2++ = fv;
-        } break;
-    case 4: {
-        unsigned int fv = *((unsigned int*)fill);
-        unsigned int* p4 = (unsigned int*)fillchunk;
-        for(i=0;i<chunksize;i+=typesize) *p4++ = fv;
-        } break;
-    case 8: {
-        unsigned long long fv = *((unsigned long long*)fill);
-        unsigned long long* p8 = (unsigned long long*)fillchunk;
-        for(i=0;i<chunksize;i+=typesize) *p8++ = fv;
-        } break;
-    default: {
-        unsigned char* p;
-        for(p=fillchunk,i=0;i<chunksize;i+=typesize,p+=typesize)
-            memcpy(p,fill,typesize);
-        } break;
-    }
-    if(fillchunkp) {*fillchunkp = fillchunk; fillchunk = NULL;}
-    nullfree(fillchunk);
-    return NC_NOERR;
-}
-    
-/**************************************************/
-/* S3 utilities */
-
-EXTERNL int
-NCZ_s3urlprocess(NCURI* url, ZS3INFO* s3)
-{
-    int stat = NC_NOERR;
-    NClist* segments = NULL;
-    NCbytes* buf = ncbytesnew();
-
-    if(url == NULL)
-        {stat = NC_EURL; goto done;}
-    /* do some verification */
-    if(strcmp(url->protocol,"https") != 0)
-        {stat = NC_EURL; goto done;}
-
-    /* Path better look absolute */
-    if(!nczm_isabsolutepath(url->path))
-    	{stat = NC_EURL; goto done;}
-
-    /* Distinguish path-style from virtual-host style from other:
-       Virtual: https://bucket-name.s3.Region.amazonaws.com/<root>
-       Path: https://s3.Region.amazonaws.com/bucket-name/<root>
-       Other: https://<host>/bucketname/<root>
-    */
-    if(url->host == NULL || strlen(url->host) == 0)
-        {stat = NC_EURL; goto done;}
-    if(endswith(url->host,AWSHOST)) { /* Virtual or path */
-        segments = nclistnew();
-        /* split the hostname by "." */
-        if((stat = nczm_split_delim(url->host,'.',segments))) goto done;
-	switch (nclistlength(segments)) {
-	default: stat = NC_EURL; goto done;
-	case 4:
-            if(strcasecmp(nclistget(segments,0),"s3")!=0)
-	        {stat = NC_EURL; goto done;}
-	    s3->urlformat = UF_PATH; 
-	    s3->region = strdup(nclistget(segments,1));
-	    break;
-	case 5:
-            if(strcasecmp(nclistget(segments,1),"s3")!=0)
-	        {stat = NC_EURL; goto done;}
-	    s3->urlformat = UF_VIRTUAL;
-	    s3->region = strdup(nclistget(segments,2));
-    	    s3->bucket = strdup(nclistget(segments,0));
-	    break;
-	}
-	/* Rebuild host to look like path-style */
-	ncbytescat(buf,"s3.");
-	ncbytescat(buf,s3->region);
-	ncbytescat(buf,AWSHOST);
-        s3->host = ncbytesextract(buf);
-    } else {
-        s3->urlformat = UF_OTHER;
-        if((s3->host = strdup(url->host))==NULL)
-	    {stat = NC_ENOMEM; goto done;}
-    }
-    /* Do fixups to make everything look like it was path style */
-    switch (s3->urlformat) {
-    case UF_PATH:
-    case UF_OTHER:
-	/* We have to process the path to get the bucket, and remove it in the path */
-	if(url->path != NULL && strlen(url->path) > 0) {
-            /* split the path by "/"  */
-   	    nclistfreeall(segments);
-	    segments = nclistnew();
-            if((stat = nczm_split_delim(url->path,'/',segments))) goto done;
-	    if(nclistlength(segments) == 0)
-	    	{stat = NC_EURL; goto done;}
-	    s3->bucket = ((char*)nclistremove(segments,0));
-	    if(nclistlength(segments) > 0) {
-	        if((stat = nczm_join(segments,&s3->rootkey))) goto done;
-	    } else
-	    	s3->rootkey = NULL;
-	    nclistfreeall(segments); segments = NULL;
-	}
-	break;
-    case UF_VIRTUAL:
-	if(url->path == NULL || strlen(url->path) == 0)
-	    s3->rootkey = NULL;
-        else
-	    s3->rootkey = strdup(url->path);
-	break;
-    default: stat = NC_EURL; goto done;
-    }
-    
-done:
-    ncbytesfree(buf);
-    nclistfreeall(segments);
-    return stat;
-}
-
-int
-NCZ_s3clear(ZS3INFO* s3)
-{
-    if(s3) {
-	nullfree(s3->host);
-	nullfree(s3->region);
-	nullfree(s3->bucket);
-	nullfree(s3->rootkey);
-    }
-    return NC_NOERR;
-}
-
-static int
-endswith(const char* s, const char* suffix)
-{
-    ssize_t ls, lsf, delta;
-    if(s == NULL || suffix == NULL) return 0;
-    ls = strlen(s);
-    lsf = strlen(suffix);
-    delta = (ls - lsf);
-    if(delta < 0) return 0;
-    if(memcmp(s+delta,suffix,lsf)!=0) return 0;
-    return 1;
-}
-
 int
 NCZ_ischunkname(const char* name,char dimsep)
 {
@@ -901,7 +930,7 @@ NCZ_ischunkname(const char* name,char dimsep)
 }
 
 char*
-NCZ_chunkpath(struct ChunkKey key,char dimsep)
+NCZ_chunkpath(struct ChunkKey key)
 {
     size_t plen = nulllen(key.varkey)+1+nulllen(key.chunkkey);
     char* path = (char*)malloc(plen+1);
@@ -913,3 +942,171 @@ NCZ_chunkpath(struct ChunkKey key,char dimsep)
     strlcat(path,key.chunkkey,plen+1);
     return path;    
 }
+
+int
+NCZ_reclaim_fill_value(NC_VAR_INFO_T* var)
+{
+    int stat = NC_NOERR;
+    if(var->fill_value) {
+	int ncid = var->container->nc4_info->controller->ext_ncid;
+	int tid = var->type_info->hdr.id;
+	stat = nc_reclaim_data_all(ncid,tid,var->fill_value,1);
+	var->fill_value = NULL;
+    }
+    /* Reclaim any existing fill_chunk */
+    if(!stat) stat = NCZ_reclaim_fill_chunk(((NCZ_VAR_INFO_T*)var->format_var_info)->cache);
+    return stat;
+}
+
+int
+NCZ_copy_fill_value(NC_VAR_INFO_T* var, void**  dstp)
+{
+    int stat = NC_NOERR;
+    int ncid = var->container->nc4_info->controller->ext_ncid;
+    int tid = var->type_info->hdr.id;
+    void* dst = NULL;
+
+    if(var->fill_value) {
+	if((stat = nc_copy_data_all(ncid,tid,var->fill_value,1,&dst))) goto done;
+    }
+    if(dstp) {*dstp = dst; dst = NULL;}
+done:
+    if(dst) (void)nc_reclaim_data_all(ncid,tid,dst,1);
+    return stat;
+}
+
+
+/* Get max str len for a variable or grp */
+/* Has side effect of setting values in the
+   internal data structures */
+int
+NCZ_get_maxstrlen(NC_OBJ* obj)
+{
+    int maxstrlen = 0;
+    assert(obj->sort == NCGRP || obj->sort == NCVAR);
+    if(obj->sort == NCGRP) {
+        NC_GRP_INFO_T* grp = (NC_GRP_INFO_T*)obj;
+	NC_FILE_INFO_T* file = grp->nc4_info;
+	NCZ_FILE_INFO_T* zfile = (NCZ_FILE_INFO_T*)file->format_file_info;
+	if(zfile->default_maxstrlen == 0)
+	    zfile->default_maxstrlen = NCZ_MAXSTR_DEFAULT;
+	maxstrlen = zfile->default_maxstrlen;
+    } else { /*(obj->sort == NCVAR)*/
+        NC_VAR_INFO_T* var = (NC_VAR_INFO_T*)obj;
+	NCZ_VAR_INFO_T* zvar = (NCZ_VAR_INFO_T*)var->format_var_info;
+        if(zvar->maxstrlen == 0)
+	    zvar->maxstrlen = NCZ_get_maxstrlen((NC_OBJ*)var->container);
+	maxstrlen = zvar->maxstrlen;
+    }
+    return maxstrlen;
+}
+
+int
+NCZ_fixed2char(const void* fixed, char** charp, size_t count, int maxstrlen)
+{
+    size_t i;
+    unsigned char* sp = NULL;
+    const unsigned char* p = fixed;
+    memset((void*)charp,0,sizeof(char*)*count);
+    for(i=0;i<count;i++,p+=maxstrlen) {
+	if(p[0] == '\0') {
+	    sp = NULL;
+	} else {
+	    if((sp = (unsigned char*)malloc(maxstrlen+1))==NULL) /* ensure null terminated */
+	        return NC_ENOMEM; 
+	    memcpy(sp,p,maxstrlen);
+	    sp[maxstrlen] = '\0';
+	}
+	charp[i] = sp;
+	sp = NULL;
+    }
+    return NC_NOERR;
+}
+
+int
+NCZ_char2fixed(const char** charp, void* fixed, size_t count, int maxstrlen)
+{
+    size_t i;
+    unsigned char* p = fixed;
+    memset(fixed,0,maxstrlen*count); /* clear target */
+    for(i=0;i<count;i++,p+=maxstrlen) {
+	size_t len;
+	if(charp[i] != NULL) {
+	    len = strlen(charp[i]);
+	    if(len > maxstrlen) len = maxstrlen;
+	    memcpy(p,charp[i],len);
+	} else {
+	    memset(p,'\0',maxstrlen);
+	}
+    }
+    return NC_NOERR;
+}
+
+/*
+Wrap NC_copy_data, but take string value into account when overwriting
+*/
+int
+NCZ_copy_data(NC_FILE_INFO_T* file, NC_TYPE_INFO_T* xtype, const void* memory, size_t count, int noclear, void* copy)
+{
+    if(xtype->hdr.id == NC_STRING && !noclear) {
+	size_t i;
+	char** scopy = (char**)copy;
+	/* Reclaim any string fill values in copy */
+	for(i=0;i<count;i++) {
+	    nullfree(scopy[i]);
+	    scopy[i] = NULL;
+	}
+    }
+    return nc_copy_data(file->controller->ext_ncid,xtype->hdr.id,memory,count,copy);
+}
+
+#if 0
+/* Recursive helper */
+static int
+checksimplejson(NCjson* json, int depth)
+{
+    int i;
+
+    switch (NCJsort(json)) {
+    case NCJ_ARRAY:
+	if(depth > 0) return 0;  /* e.g. [...,[...],...]  or [...,{...},...] */
+	for(i=0;i < NCJlength(json);i++) {
+	    NCjson* j = NCJith(json,i);
+	    if(!checksimplejson(j,depth+1)) return 0;
+        }
+	break;
+    case NCJ_DICT:
+    case NCJ_NULL:
+    case NCJ_UNDEF:
+	return 0;
+    default: break;
+    }
+    return 1;
+}
+#endif
+
+/* Return 1 if the attribute will be stored as a complex JSON valued attribute; return 0 otherwise */
+int
+NCZ_iscomplexjson(NCjson* json, nc_type typehint)
+{
+    int i, stat = 0;
+
+    switch (NCJsort(json)) {
+    case NCJ_ARRAY:
+	/* If the typehint is NC_CHAR, then always treat it as complex */
+	if(typehint == NC_CHAR) {stat = 1; goto done;}
+	/* Otherwise see if it is a simple vector of atomic values */
+	for(i=0;i < NCJlength(json);i++) {
+	    NCjson* j = NCJith(json,i);
+	    if(!NCJisatomic(j)) {stat = 1; goto done;}
+        }
+	break;
+    case NCJ_DICT:
+    case NCJ_NULL:
+    case NCJ_UNDEF:
+	stat = 1; goto done;
+    default: break;
+    }
+done:
+    return stat;
+}
