@@ -1,4 +1,4 @@
-/* Copyright 1988-2010 University Corporation for Atmospheric Research
+/* Copyright 1988-2018 University Corporation for Atmospheric Research
    See netcdf/COPYRIGHT file for copying and redistribution
    conditions.
 
@@ -7,7 +7,7 @@
    for consistency.  Leaves the file around afterwards.
 
    Based on a program to test the nasa look-alike program, so not the
-   most appropropriate test. See ../nctest for a complete spec test.
+   most appropriate test. See ../nctest for a complete spec test.
 */
 
 #define REDEF
@@ -353,19 +353,19 @@ main(int argc, char *argv[])
 	size_t ui;
 	const struct tcdfvar *tvp = testvars;
 	union getret got;
-	const size_t initialsz = 8192;
-	size_t chunksz = 8192;
 	size_t align = 8192/32;
 
 #ifdef USE_PNETCDF
 	MPI_Init(&argc, &argv);
 
-    cmode |= (NC_PNETCDF);
-#ifdef USE_CDF5
+#ifdef ENABLE_CDF5
 	cmode |= (NC_64BIT_DATA);
 #endif
 	ret = nc_create_par(fname,cmode, MPI_COMM_WORLD, MPI_INFO_NULL, &id);
 #else
+	const size_t initialsz = 8192;
+	size_t chunksz = 8192;
+
 	ret = nc__create(fname,cmode, initialsz, &chunksz, &id);
 	if(ret != NC_NOERR) {
  		/* (void) fprintf(stderr, "trying again\n"); */
@@ -475,7 +475,6 @@ main(int argc, char *argv[])
  */
         omode = NC_NOWRITE;
 #ifdef USE_PNETCDF
-        omode |= NC_PNETCDF;
 	ret = nc_open_par(fname,omode, MPI_COMM_WORLD, MPI_INFO_NULL, &id);
 #else
 	ret = nc__open(fname,omode, &chunksz, &id);
