@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *   $Header: /upc/share/CVS/netcdf-3/ncdump/ncdump.h,v 1.34 2009/12/02 20:03:43 russ Exp $
  *********************************************************************/
@@ -43,7 +43,10 @@ typedef struct {			/* specification for how to format dump */
 				 * for optimization characteristics:
 				 * _Compression, _Chunking,
 				 * _Endianness, _Format, _Checksum,
-				 * _NoFill */
+				 * _NoFill, _NetCDF4 */
+
+    bool_t filter_atts;		/* if true, output filters using _Filter in place of
+				 * _Fletcher32, _Shuffle, and _Deflate. */
 
     Nclang data_lang;		/* Specifies index conventions used in data
 				 * comments, either LANG_C (C, 0-based,
@@ -78,6 +81,7 @@ typedef struct {			/* specification for how to format dump */
     int nc_mode;                /* mode as reported by inq_format_extended */
 
     int xopt_inmemory;      /* Use in-memory option; testing only */
+    int xopt_props ;      /* 1=>Unconditionally Suppress properties attribute */
 } fspec_t;
 
 #endif	/*_NCDUMP_H_ */
