@@ -70,7 +70,7 @@ cleanup() {
 # remove all created files
 reset() {
     cleanup
-    rm -fr tst_nc5.nc tst_nc5.cdl
+    rm -fr tst_nc5.nc tst_nc5.cdl tmp_ncc5.cdl
     rm -f tst_nc5_omit.nc tst_nc5_omit.cdl
 }
 
@@ -79,21 +79,21 @@ reset
 if test "x$T1" = x1 ; then
 
 # Create a simple classic input file 
-./tst_chunking tst_nc5.nc
+${execdir}/tst_chunking tmp_nc5_base.nc
 
 # Save a .cdl version
-${NCDUMP} tst_nc5.nc > tst_nc5.cdl
+${NCDUMP} tmp_nc5_base.nc > tmp_nc5_base.cdl
 
 echo "*** Test nccopy -c with per-variable chunking; classic->enhanced"
 # This should produce same as -c dim0/,dim1/1,dim2/,dim3/1,dim4/,dim5/1,dim6/
 # But note that the chunk product is less than default, so we need to reduce it (-M)
-${NCCOPY} -M1000 -c ivar:7,1,2,1,5,1,9 tst_nc5.nc tmp_nc5.nc
-${NCDUMP} -n tst_nc5 tmp_nc5.nc > tmp_nc5.cdl
+${NCCOPY} -M1000 -c ivar:7,1,2,1,5,1,9 tmp_nc5_base.nc tmp_nc5.nc
+${NCDUMP} -n tmp_nc5_base tmp_nc5.nc > tmp_nc5.cdl
 # Verify that the core cdl is the same
-diff tst_nc5.cdl tmp_nc5.cdl
+diff tmp_nc5_base.cdl tmp_nc5.cdl
 # Look at the output chunking of ivar
 rm -f tmp_nc5a.cdl # reuse
-${NCDUMP} -hs -n tst_nc5 tmp_nc5.nc > tmp_nc5.cdl
+${NCDUMP} -hs -n tmp_nc5_base tmp_nc5.nc > tmp_nc5.cdl
 # extract the chunking line
 TESTLINE=`sed -e '/ivar:_ChunkSizes/p' -e d <tmp_nc5.cdl`
 # track line to match
@@ -108,16 +108,16 @@ if test "x$T2" = x1 ; then
 
 echo "*** Test nccopy -c with per-variable chunking; enhanced->enhanced"
 reset
-./tst_chunking tst_nc5.nc deflate
-${NCDUMP} -n tst_nc5 tst_nc5.nc > tst_nc5.cdl
+${execdir}/tst_chunking tst_nc5.nc deflate
+${NCDUMP} -n tmp_nc5_base tst_nc5.nc > tst_nc5.cdl
 # Use -M to ensure that chunking takes effect
 ${NCCOPY} -M500 -c ivar:4,1,2,1,5,2,3 tst_nc5.nc tmp_nc5.nc
-${NCDUMP} -n tst_nc5 tmp_nc5.nc > tmp_nc5.cdl
+${NCDUMP} -n tmp_nc5_base tmp_nc5.nc > tmp_nc5.cdl
 diff tst_nc5.cdl tmp_nc5.cdl
 
 # Look at the output chunking
 rm -f tmp_nc5.cdl # reuse
-${NCDUMP} -hs -n tst_nc5 tmp_nc5.nc > tmp_nc5.cdl
+${NCDUMP} -hs -n tmp_nc5_base tmp_nc5.nc > tmp_nc5.cdl
 # extract the chunking line
 TESTLINE=`sed -e '/ivar:_ChunkSizes/p' -e d <tmp_nc5.cdl`
 # track line to match
@@ -132,14 +132,14 @@ if test "x$T3" = x1 ; then
 
 echo "*** Test nccopy -c with FQN var name; enhanced ->enhanced"
 reset
-./tst_chunking tst_nc5.nc group
-${NCDUMP} -n tst_nc5 tst_nc5.nc > tst_nc5.cdl
+${execdir}/tst_chunking tst_nc5.nc group
+${NCDUMP} -n tmp_nc5_base tst_nc5.nc > tst_nc5.cdl
 ${NCCOPY} -M500 -c /g/ivar:4,1,2,1,5,2,3 tst_nc5.nc tmp_nc5.nc
-${NCDUMP} -n tst_nc5 tmp_nc5.nc > tmp_nc5.cdl
+${NCDUMP} -n tmp_nc5_base tmp_nc5.nc > tmp_nc5.cdl
 diff tst_nc5.cdl tmp_nc5.cdl
 
 # Verify chunking
-${NCDUMP} -hs -n tst_nc5 tmp_nc5.nc > tmp_nc5.cdl
+${NCDUMP} -hs -n tmp_nc5_base tmp_nc5.nc > tmp_nc5.cdl
 # extract the chunking line
 TESTLINE=`sed -e '/ivar:_ChunkSizes/p' -e d <tmp_nc5.cdl`
 # track line to match
@@ -152,14 +152,14 @@ if test "x$T4" = x1 ; then
 
 echo "*** Test nccopy -c with unlimited dimension; classic ->enhanced"
 reset
-./tst_chunking tst_nc5.nc unlimited # should produce modified tmp_nc5.nc with ivar of rank 2
-${NCDUMP} -n tst_nc5 tst_nc5.nc > tst_nc5.cdl
+${execdir}/tst_chunking tst_nc5.nc unlimited # should produce modified tmp_nc5.nc with ivar of rank 2
+${NCDUMP} -n tmp_nc5_base tst_nc5.nc > tst_nc5.cdl
 ${NCCOPY} -M500 -c ivar:5,3 tst_nc5.nc tmp_nc5.nc
-${NCDUMP} -n tst_nc5 tmp_nc5.nc > tmp_nc5.cdl
+${NCDUMP} -n tmp_nc5_base tmp_nc5.nc > tmp_nc5.cdl
 diff tst_nc5.cdl tmp_nc5.cdl
 
 # Verify chunking
-${NCDUMP} -hs -n tst_nc5 tmp_nc5.nc > tmp_nc5.cdl
+${NCDUMP} -hs -n tmp_nc5_base tmp_nc5.nc > tmp_nc5.cdl
 # extract the chunking line
 TESTLINE=`sed -e '/ivar:_ChunkSizes/p' -e d <tmp_nc5.cdl`
 # track line to match
@@ -175,7 +175,7 @@ if test "x$T5" = x1 ; then
 
 echo "*** Test nccopy -c fvar: to suppress chunking; classic ->enhanced"
 reset
-./tst_chunking tst_nc5_omit.nc
+${execdir}/tst_chunking tst_nc5_omit.nc
 ${NCDUMP} -n tst_nc5_omit tst_nc5_omit.nc > tst_nc5_omit.cdl
 ${NCCOPY} -M500 -c ivar:7,1,2,1,5,1,9 -c fvar: tst_nc5_omit.nc tmp_nc5_omit.nc
 ${NCDUMP} -n tst_nc5_omit tmp_nc5_omit.nc > tmp_nc5_omit.cdl
