@@ -1,4 +1,4 @@
-/* Copyright 2009, UCAR/Unidata and OPeNDAP, Inc.
+/* Copyright 2018, UCAR/Unidata and OPeNDAP, Inc.
    See the COPYRIGHT file for more information. */
 
 #ifndef OCOCDBG_H
@@ -58,9 +58,6 @@ about how IO is getting along.
 */
 #undef OCPROGRESS
 
-extern int ocdebug;
-extern int cedebug;
-
 /*extern char* dent2(int n);*/
 /*/extern char* dent(int n);*/
 extern int ocpanic(const char* fmt, ...);
@@ -103,4 +100,3 @@ extern CURLcode ocreportcurlerror(struct OCstate* state, CURLcode cstat);
 #define OCTHROWCHK(e) OCCATCHCHK(e)
 
 #endif /*OCOCDBG_H*/
-
